# # -*- coding: utf-8 -*-
from __future__ import print_function, unicode_literals
import os
import unittest
import shutil
import stat
import tempfile
try:
    import configparser
except ImportError:
    import ConfigParser as configparser

from preupg.xmlgen.compose import ComposeXML
from preupg import xccdf
from preupg import settings
from preupg.xmlgen.xml_utils import XmlUtils
from preupg.xmlgen.oscap_group_xml import OscapGroupXml
from preupg.utils import FileHelper
from preupg.xml_manager import html_escape
try:
    import base
except ImportError:
    import tests.base as base


class TestXMLCompose(base.TestCase):

    """Tests of right composing of contents in groups."""

    result_dir = None
    target_tree = None
    tree = None

    def setUp(self):
        self.temp_dir = tempfile.mkdtemp(prefix='preupgrade', dir='/tmp')
        self.result_dir = os.path.join(self.temp_dir, 'FOOBAR6_7' +
                                       settings.results_postfix)
        dir_name = os.path.join(os.getcwd(), 'tests', 'FOOBAR6_7')
        shutil.copytree(dir_name, self.result_dir)

        self.autocomplete_orig = settings.autocomplete
        settings.autocomplete = False
        self.data_dir_orig = settings.data_dir
        settings.data_dir = os.path.join(os.getcwd(), "data")
        self.upgrade_path_orig = settings.UPGRADE_PATH
        self.target_tree = ComposeXML.run_compose(self.result_dir)

    def tearDown(self):
        shutil.rmtree(self.result_dir)
        settings.autocomplete = self.autocomplete_orig
        settings.data_dir = self.data_dir_orig
        settings.UPGRADE_PATH = self.upgrade_path_orig
        pass

    def test_compose(self):
        """Basic test of composing"""
        expected_groups = ['failed', 'fixed', 'needs_action',
                           'needs_inspection', 'not_applicable', 'pass',
                           'unicode']

        generated_group = []
        for group in self.target_tree.findall(xccdf.XMLNS + "Group"):
            generated_group.append(group.get('id'))
        self.assertEqual(['xccdf_preupg_group_'+x for x in expected_groups], generated_group)

    def test_unicode_xml(self):
        """
        Test processing of non-ascii characters inside title and description
        sections.
        """
        u_title = u'Čekujeme unicode u hasičů'
        u_descr = u'Hoří horní heršpická hospoda Hrbatý hrozen.'
        uni_xml = os.path.join(self.result_dir, "unicode", "group.xml")
        try:
            # XML files should be always in utf-8!
            lines = [x.decode('utf-8') for x in FileHelper.get_file_content(uni_xml, "rb", True, False)]
        except IOError:
            assert False
        title = [x for x in lines if u_title in x]
        descr = [x for x in lines if u_descr in x]
        self.assertTrue(title, "title is wrong encoded or missing")
        self.assertTrue(descr, "description is wrong encoded or missing")

    def test_unicode_script_author(self):
        """Test processing of non-ascii characters for author section"""
        u_author = b'Petr Stod\xc5\xaflka'.decode(settings.defenc)
        script_file = os.path.join(self.result_dir, "unicode", "check")
        settings.autocomplete = True
        self.target_tree = None
        try:
            self.target_tree = ComposeXML.run_compose(self.result_dir)
        except UnicodeEncodeError:
            # TODO This has to be fixed for all supported Python versions like Python3.5,2.7 and 2.6
            assert True
        self.assertTrue(self.target_tree)
        try:
            lines = FileHelper.get_file_content(script_file, "rb", True)
        except IOError:
            assert False
        author = [x for x in lines if u_author in x]
        self.assertTrue(author)


class TestScriptGenerator(base.TestCase):

    """Main testing of right generating of XML files for OSCAP."""
    dirname = None
    filename = None
    test_solution = None
    loaded_ini = None
    check_script = None
    check_sh = None
    solution_text = None
    rule = None
    xml_utils = None
    test_ini = []

    def setUp(self):
        self.root_dir_name = "tests/FOOBAR6_7" + settings.results_postfix
        self.dirname = os.path.join(self.root_dir_name, "test")
        if os.path.exists(self.dirname):
            shutil.rmtree(self.dirname)
        os.makedirs(self.dirname)
        self.filename = os.path.join(self.dirname, 'test.ini')
        self.rule = []
        self.loaded_ini = {}
        self.test_ini = {'content_title': 'Testing content title',
                         'content_description': ' some content description',
                         'author': 'test <test@redhat.com>',
                         'config_file': '/etc/named.conf'
                         }
        self.check_sh = """#!/bin/bash

#END GENERATED SECTION

#This is testing check script
 """
        check_name = os.path.join(self.dirname, settings.check_script)
        FileHelper.write_to_file(check_name, "wb", self.check_sh)
        os.chmod(check_name, stat.S_IEXEC | stat.S_IRWXG | stat.S_IRWXU)

        self.solution_text = """
A solution text for test suite"
"""
        test_solution_name = os.path.join(self.dirname, settings.solution_txt)
        FileHelper.write_to_file(test_solution_name, "wb", self.solution_text)
        os.chmod(check_name, stat.S_IEXEC | stat.S_IRWXG | stat.S_IRWXU)

    def _return_check(self, text):
        content = FileHelper.get_file_content(os.path.join(
            self.dirname, settings.check_script), "rb", method=True)
        found = [x for x in content if x.startswith(text)]
        return found

    def test_applies_to(self):
        self.test_ini['applies_to'] = 'test_rpm'
        self.loaded_ini[self.filename] = self.test_ini
        self.xml_utils = XmlUtils(self.root_dir_name, self.dirname,
                                  self.loaded_ini)
        self.rule = self.xml_utils.prepare_sections()
        self.assertTrue(self._return_check('check_applies_to "test_rpm"'))

    def test_check_bin(self):
        self.test_ini['binary_req'] = 'cpf'
        self.loaded_ini[self.filename] = self.test_ini
        self.xml_utils = XmlUtils(self.root_dir_name, self.dirname,
                                  self.loaded_ini)
        self.rule = self.xml_utils.prepare_sections()
        self.assertTrue(self._return_check('check_rpm_to "" "cpf"'))

    def test_check_rpm(self):
        self.test_ini['requires'] = 'test_rpm'
        self.loaded_ini[self.filename] = self.test_ini
        self.xml_utils = XmlUtils(self.root_dir_name, self.dirname,
                                  self.loaded_ini)
        self.rule = self.xml_utils.prepare_sections()
        self.assertTrue(self._return_check('check_rpm_to "test_rpm" ""'))

    def test_check_rpm_bin(self):
        self.test_ini['binary_req'] = 'cpf'
        self.test_ini['requires'] = 'test_rpm'
        self.loaded_ini[self.filename] = self.test_ini
        self.xml_utils = XmlUtils(self.root_dir_name, self.dirname,
                                  self.loaded_ini)
        self.rule = self.xml_utils.prepare_sections()
        self.assertTrue(self._return_check('check_rpm_to "test_rpm" "cpf"'))

    def test_applies_to_bin(self):
        self.test_ini['applies_to'] = 'test_rpm'
        self.test_ini['binary_req'] = 'cpf'
        self.loaded_ini[self.filename] = self.test_ini
        self.xml_utils = XmlUtils(self.root_dir_name, self.dirname,
                                  self.loaded_ini)
        self.rule = self.xml_utils.prepare_sections()
        self.assertTrue(self._return_check('check_applies_to "test_rpm"'))
        self.assertTrue(self._return_check('check_rpm_to "" "cpf"'))

    def tearDown(self):
        shutil.rmtree(self.dirname)


class TestXML(base.TestCase):

    """Main testing of right generating of XML files for OSCAP."""
    dirname = None
    filename = None
    test_solution = None
    loaded_ini = None
    check_script = None
    check_sh = None
    solution_text = None
    rule = None
    xml_utils = None

    def setUp(self):
        self.root_dir_name = "tests/FOOBAR6_7" + settings.results_postfix
        self.dirname = os.path.join(self.root_dir_name, "test")
        if os.path.exists(self.dirname):
            shutil.rmtree(self.dirname)
        os.makedirs(self.dirname)
        self.filename = os.path.join(self.dirname, 'test.ini')
        self.rule = []
        self.loaded_ini = {}
        test_ini = {'content_title': 'Testing content title',
                    'content_description': ' some content description',
                    'author': 'test <test@redhat.com>',
                    'config_file': '/etc/named.conf',
                    'applies_to': 'test',
                    'requires': 'bash',
                    'binary_req': 'sed'}
        self.loaded_ini[self.filename] = test_ini
        self.check_sh = """#!/bin/bash

#END GENERATED SECTION

#This is testing check script
 """
        check_name = os.path.join(self.dirname, settings.check_script)
        FileHelper.write_to_file(check_name, "wb", self.check_sh)
        os.chmod(check_name, stat.S_IEXEC | stat.S_IRWXG | stat.S_IRWXU)

        self.solution_text = """
A solution text for test suite"
"""
        test_solution_name = os.path.join(self.dirname, settings.solution_txt)
        FileHelper.write_to_file(test_solution_name, "wb", self.solution_text)
        os.chmod(check_name, stat.S_IEXEC | stat.S_IRWXG | stat.S_IRWXU)
        self.xml_utils = XmlUtils(self.root_dir_name, self.dirname,
                                  self.loaded_ini)
        self.rule = self.xml_utils.prepare_sections()

    def tearDown(self):
        shutil.rmtree(self.dirname)
        if os.path.exists(os.path.join(os.getcwd(), 'migrate')):
            os.unlink(os.path.join(os.getcwd(), 'migrate'))
        if os.path.exists(os.path.join(os.getcwd(), 'upgrade')):
            os.unlink(os.path.join(os.getcwd(), 'upgrade'))

    def test_group_xml(self):
        """Basic test for whole program"""
        self.assertTrue(self.loaded_ini[self.filename])
        self.assertTrue(self.rule)

    def test_xml_rule_id(self):
        rule_id = [x for x in self.rule if '<Rule id="xccdf_preupg_rule_test_check" selected="true">' in x]
        self.assertTrue(rule_id)

    def test_xml_profile_id(self):
        profile = [x for x in self.rule if '<Profile id="xccdf_preupg_profile_default">' in x]
        self.assertTrue(profile)

    def test_xml_rule_title(self):
        rule_title = [x for x in self.rule if "<title>Testing content title</title>" in x]
        self.assertTrue(rule_title)

    def test_xml_config_file(self):
        conf_file = [x for x in self.rule if "<xhtml:li>/etc/named.conf</xhtml:li>" in x]
        self.assertTrue(conf_file)

    def test_xml_fix_text(self):
        fix_text = [x for x in self.rule if "<fixtext>_test_SOLUTION_MSG</fixtext>" in x]
        self.assertTrue(fix_text)

    def test_xml_solution_type_text(self):
        self.xml_utils = XmlUtils(self.root_dir_name, self.dirname,
                                  self.loaded_ini)
        self.rule = self.xml_utils.prepare_sections()
        fix_text = [x for x in self.rule if "<fixtext>_test_SOLUTION_MSG</fixtext>" in x]
        self.assertTrue(fix_text)


    def test_check_script_author(self):
        settings.autocomplete = True
        self.rule = self.xml_utils.prepare_sections()
        lines = FileHelper.get_file_content(os.path.join(
            self.dirname, settings.check_script), "rb", method=True)
        author = [x for x in lines if "test <test@redhat.com>" in x]
        self.assertTrue(author)

    def test_xml_check_export_tmp_preupgrade(self):
        self.rule = self.xml_utils.prepare_sections()
        check_export = [x for x in self.rule if 'xccdf_preupg_value_tmp_preupgrade' in x]
        self.assertTrue(check_export)

    def test_xml_current_directory(self):
        self.rule = self.xml_utils.prepare_sections()
        cur_directory = [x for x in self.rule if '<check-export export-name="CURRENT_DIRECTORY" value-id="xccdf_preupg_value_test_check_state_current_directory" />' in x]
        self.assertTrue(cur_directory)

    def _create_temporary_dir(self):
        settings.UPGRADE_PATH = tempfile.mkdtemp()
        if os.path.exists(settings.UPGRADE_PATH):
            shutil.rmtree(settings.UPGRADE_PATH)
        os.makedirs(settings.UPGRADE_PATH)
        migrate = os.path.join(settings.UPGRADE_PATH, 'migrate')
        upgrade = os.path.join(settings.UPGRADE_PATH, 'upgrade')
        return migrate, upgrade

    def _delete_temporary_dir(self, migrate, upgrade):
        if os.path.exists(migrate):
            os.unlink(migrate)
        if os.path.exists(upgrade):
            os.unlink(upgrade)
        if os.path.exists(settings.UPGRADE_PATH):
            shutil.rmtree(settings.UPGRADE_PATH)

    def test_xml_migrate_not_upgrade(self):
        test_ini = {'content_title': 'Testing only migrate title',
                    'content_description': ' some content description',
                    'author': 'test <test@redhat.com>',
                    'config_file': '/etc/named.conf',
                    'applies_to': 'test',
                    'requires': 'bash',
                    'binary_req': 'sed',
                    'mode': 'migrate'}
        ini = {}
        old_settings = settings.UPGRADE_PATH
        migrate, upgrade = self._create_temporary_dir()
        ini[self.filename] = test_ini
        xml_utils = XmlUtils(self.root_dir_name, self.dirname, ini)
        xml_utils.prepare_sections()
        migrate_file = FileHelper.get_file_content(migrate, 'rb', method=True)
        tag = [x.strip() for x in migrate_file if 'xccdf_preupg_rule_test_check_script' in x.strip()]
        self.assertIsNotNone(tag)
        try:
            upgrade_file = FileHelper.get_file_content(upgrade, 'rb', method=True)
        except IOError:
            upgrade_file = None
        self.assertIsNone(upgrade_file)
        self._delete_temporary_dir(migrate, upgrade)
        settings.UPGRADE_PATH = old_settings

    def test_xml_upgrade_not_migrate(self):
        test_ini = {'content_title': 'Testing only migrate title',
                    'content_description': ' some content description',
                    'author': 'test <test@redhat.com>',
                    'config_file': '/etc/named.conf',
                    'applies_to': 'test',
                    'requires': 'bash',
                    'binary_req': 'sed',
                    'mode': 'upgrade'}
        ini = {}
        old_settings = settings.UPGRADE_PATH
        migrate, upgrade = self._create_temporary_dir()
        ini[self.filename] = test_ini
        xml_utils = XmlUtils(self.root_dir_name, self.dirname, ini)
        xml_utils.prepare_sections()
        upgrade_file = FileHelper.get_file_content(upgrade, 'rb', method=True)
        tag = [x.strip() for x in upgrade_file if 'xccdf_preupg_rule_test_check_script' in x.strip()]
        self.assertIsNotNone(tag)
        try:
            migrate_file = FileHelper.get_file_content(migrate, 'rb', method=True)
        except IOError:
            migrate_file = None
        self.assertIsNone(migrate_file)
        self._delete_temporary_dir(migrate, upgrade)
        settings.UPGRADE_PATH = old_settings

    def test_xml_migrate_and_upgrade(self):
        test_ini = {'content_title': 'Testing only migrate title',
                    'content_description': ' some content description',
                    'author': 'test <test@redhat.com>',
                    'config_file': '/etc/named.conf',
                    'applies_to': 'test',
                    'requires': 'bash',
                    'binary_req': 'sed',
                    'mode': 'migrate, upgrade'}
        ini = {}
        old_settings = settings.UPGRADE_PATH
        migrate, upgrade = self._create_temporary_dir()
        ini[self.filename] = test_ini
        xml_utils = XmlUtils(self.root_dir_name, self.dirname, ini)
        xml_utils.prepare_sections()
        migrate_file = FileHelper.get_file_content(migrate, 'rb', method=True)
        tag = [x.strip() for x in migrate_file if 'xccdf_preupg_rule_test_check_script' in x.strip()]
        self.assertIsNotNone(tag)
        upgrade_file = FileHelper.get_file_content(upgrade, 'rb', method=True)
        tag = [x.strip() for x in upgrade_file if 'xccdf_preupg_rule_test_check_script' in x.strip()]
        self.assertIsNotNone(tag)
        self._delete_temporary_dir(migrate, upgrade)
        settings.UPGRADE_PATH = old_settings

    def test_xml_not_migrate_not_upgrade(self):
        test_ini = {'content_title': 'Testing only migrate title',
                    'content_description': ' some content description',
                    'author': 'test <test@redhat.com>',
                    'config_file': '/etc/named.conf',
                    'applies_to': 'test',
                    'requires': 'bash',
                    'binary_req': 'sed'}
        ini = {}
        old_settings = settings.UPGRADE_PATH
        migrate, upgrade = self._create_temporary_dir()
        ini[self.filename] = test_ini
        xml_utils = XmlUtils(self.root_dir_name, self.dirname, ini)
        xml_utils.prepare_sections()
        migrate_file = FileHelper.get_file_content(migrate, 'rb', method=True)
        tag = [x.strip() for x in migrate_file if 'xccdf_preupg_rule_test_check_script' in x.strip()]
        self.assertIsNotNone(tag)
        upgrade_file = FileHelper.get_file_content(upgrade, 'rb', method=True)
        tag = [x.strip() for x in upgrade_file if 'xccdf_preupg_rule_test_check_script' in x.strip()]
        self.assertIsNotNone(tag)
        self._delete_temporary_dir(migrate, upgrade)
        settings.UPGRADE_PATH = old_settings

    def test_xml_check_script_reference(self):
        self.rule = self.xml_utils.prepare_sections()
        check_script_reference = [x for x in self.rule if '<check-content-ref href="check" />' in x]
        self.assertTrue(check_script_reference)

    def test_values_id(self):
        self.rule = self.xml_utils.prepare_sections()
        value_current_dir = [x for x in self.rule if '<Value id="xccdf_preupg_value_test_check_state_current_directory"' in x]
        self.assertTrue(value_current_dir)
        value_current_dir_set = [x for x in self.rule if '<value>SCENARIO/test</value>' in x]
        self.assertTrue(value_current_dir_set)

    def test_check_script_applies_to(self):
        self.rule = self.xml_utils.prepare_sections()
        lines = FileHelper.get_file_content(os.path.join(
            self.dirname, settings.check_script), "rb", method=True)
        applies = [x for x in lines if 'check_applies_to "test"' in x]
        self.assertTrue(applies)

    def test_check_script_common(self):
        self.rule = self.xml_utils.prepare_sections()
        lines = FileHelper.get_file_content(os.path.join(
            self.dirname, settings.check_script), "rb", method=True)
        common = [x for x in lines if '. /usr/share/preupgrade/common.sh' in x]
        self.assertTrue(common)

    def test_check_script_requires(self):
        self.rule = self.xml_utils.prepare_sections()
        lines = FileHelper.get_file_content(os.path.join(
            self.dirname, settings.check_script), "rb", method=True)
        check_rpm_to = [x for x in lines if 'check_rpm_to "bash" "sed"' in x]
        self.assertTrue(check_rpm_to)


class TestIncorrectINI(base.TestCase):

    """
    Tests right processing of INI files including incorrect input which
    could make for crash with traceback.
    """
    dir_name = None
    filename = None
    rule = None
    test_solution = None
    check_script = None
    loaded_ini = {}
    test_ini = None
    xml_utils = None

    def setUp(self):
        self.root_dir_name = "tests/FOOBAR6_7"
        self.dir_name = os.path.join(self.root_dir_name, "incorrect_ini")
        os.makedirs(self.dir_name)
        self.filename = os.path.join(self.dir_name, 'test.ini')
        self.rule = []
        self.test_ini = {'content_title': 'Testing content title',
                         'content_description': 'Some content description',
                         'author': 'test <test@redhat.com>',
                         'config_file': '/etc/named.conf',
                         'applies_to': 'test'
                         }
        solution_text = """
A solution text for test suite"
"""
        check_sh = """#!/bin/bash

#END GENERATED SECTION

#This is testing check script
 """
        FileHelper.write_to_file(os.path.join(
            self.dir_name, settings.solution_txt), "wb", solution_text)
        FileHelper.write_to_file(os.path.join(
            self.dir_name, settings.check_script), "wb", check_sh)

    def tearDown(self):
        shutil.rmtree(self.dir_name)

    def test_incorrect_tag(self):
        """
        Check occurrence of incorrect tag
        Tests issue #30
        """
        text_ini = '[preupgrade]\n'
        text_ini += '\n'.join([key + " = " + self.test_ini[key] for key in self.test_ini])
        text_ini += '\n[]\neliskk\n'
        FileHelper.write_to_file(self.filename, "wb", text_ini)
        oscap = OscapGroupXml(self.root_dir_name, self.dir_name)
        self.assertRaises(configparser.ParsingError, oscap.find_all_ini)


class TestGroupXML(base.TestCase):

    """Basic test for creating group.xml file"""

    dir_name = None
    filename = None
    rule = []
    loaded_ini = {}
    xml_utils = None

    def setUp(self):
        self.root_dir_name = "tests/FOOBAR6_7"
        self.dir_name = os.path.join(self.root_dir_name, "test_group")
        os.makedirs(self.dir_name)
        self.filename = os.path.join(self.dir_name, 'group.ini')
        test_ini = {'group_title': 'Testing content title'}
        self.assertTrue(test_ini)
        self.loaded_ini[self.filename] = test_ini

    def tearDown(self):
        shutil.rmtree(self.dir_name)

    def test_group_ini(self):
        """Basic test creation group.xml file"""
        self.xml_utils = XmlUtils(self.root_dir_name, self.dir_name,
                                  self.loaded_ini)
        self.rule = self.xml_utils.prepare_sections()
        group_tag = [x for x in self.rule if '<Group id="xccdf_preupg_group_test_group" selected="true">' in x]
        self.assertTrue(group_tag)
        title_tag = [x for x in self.rule if '<title>Testing content title</title>' in x]
        self.assertTrue(title_tag)


class HTMLEscapeTest(base.TestCase):

    """Testing of right transform of unsafe characters to their entities"""

    def test_basic_string(self):
        """Test if single string is escaped well"""
        input_char = "asd < > &"
        expected_output = "asd &lt; &gt; &amp;"
        output = html_escape(input_char)
        self.assertEqual(output, expected_output)

    def test_all(self):
        """Test if list of strings is escaped well"""
        tmp_input = "a<b>x'xyz&z"
        output = html_escape(tmp_input)
        expected_ouput = 'a&lt;b&gt;x&apos;xyz&amp;z'
        self.assertEqual(output, expected_ouput)

    def test_amp_expand(self):
        """Test whether ampersand is not being expanded multiple times"""
        input_char = 'asd<>&'
        expected_output = 'asd&lt;&gt;&amp;'
        output = html_escape(input_char)
        self.assertEqual(output, expected_output)
        input_multi_char = 'asd<><>&&'
        expected_multi_output = 'asd&lt;&gt;&lt;&gt;&amp;&amp;'
        output_multi = html_escape(input_multi_char)
        self.assertEqual(output_multi, expected_multi_output)


class ComposeTest(base.TestCase):

    def setUp(self):
        pass


def suite():
    """Add classes which should be included in testing"""
    loader = unittest.TestLoader()
    suite = unittest.TestSuite()
    suite.addTest(loader.loadTestsFromTestCase(TestGroupXML))
    suite.addTest(loader.loadTestsFromTestCase(TestXML))
    suite.addTest(loader.loadTestsFromTestCase(TestIncorrectINI))
    suite.addTest(loader.loadTestsFromTestCase(TestXMLCompose))
    suite.addTest(loader.loadTestsFromTestCase(HTMLEscapeTest))
    suite.addTest(loader.loadTestsFromTestCase(TestScriptGenerator))
    return suite

if __name__ == '__main__':
    unittest.TextTestRunner(verbosity=3).run(suite())



# -*- coding: utf-8 -*-

"""
Class creates a set of packages for migration scenario
"""

import os

from preupg.utils import FileHelper
from preupg.kickstart.application import BaseKickstart
from preupg import settings


class FirewallHandling(BaseKickstart):
    """class for replacing/updating package names"""

    def __init__(self, handler):
        """
        """
        self.handler = handler
        self.ports = []
        self.services = []
        self.firewall = FirewallHandling.get_kickstart_firewall(os.path.join(settings.KS_DIR, 'firewall-cmd'))

    @staticmethod
    def get_kickstart_firewall(filename):
        """
        service=ftp service=RH-Satellite-6 service=high-availability service=bacula service=vdsm
        service=radius service=ssh service=dns service=mdns service=ipp-client service=ipp service=https
        service=kerberos service=ntp service=kpasswd service=ldap service=ldaps

        port=20:tcp port=49:tcp port=49:udp

        returns dictionary with names and URLs
        :param filename: filename with available-repos
        :return: dictionary with enabled repolist
        """
        try:
            lines = FileHelper.get_file_content(filename, 'rb', method=True)
        except IOError:
            return None
        lines = [x for x in lines if not x.startswith('#') and not x.startswith(' ')]
        if not lines:
            return None
        return lines

    @staticmethod
    def get_services(line, name):
        services = []
        for serv in line.split():
            if serv.startswith(name):
                try:
                    fields = serv.split('=')
                    services.append(fields[1])
                except KeyError:
                    pass
        return services

    @staticmethod
    def get_ports(line, name):
        services = []
        for serv in line.split():
            if serv.startswith(name):
                try:
                    fields = serv.split('=')
                    services.append(fields[1])
                except KeyError:
                    pass
        return services

    def get_firewall_data(self):
        port = 'port'
        service = 'service'
        if self.firewall:
            for line in self.firewall:
                if line.startswith(service):
                    self.services = FirewallHandling.get_services(line, service)
                if line.startswith(port):
                    self.ports = FirewallHandling.get_ports(line, port)

    def update_firewall(self):
        if self.firewall is None:
            return
        if self.services or self.ports:
            self.handler.enabled = True
            if self.services:
                self.handler.firewall.services = self.services
            if self.ports:
                self.handler.firewall.ports = self.ports

    def run_module(self, *args, **kwargs):
        self.get_firewall_data()
        self.update_firewall()



import abc
from functools import wraps
import unittest

"""
Use this module to subclass and enhance upstream classes
with test-suite-specific features like custom asserts
for unittest.TestCase.
"""


class TestCase(unittest.TestCase):

    # ported from 2.7's unittest
    def assertIsNone(self, obj, msg=None):
        """Same as self.assertTrue(obj is None), with a nicer
        default message."""
        if obj is not None:
            standardMsg = '%r is not None' % obj
            self.fail(self._formatMessage(msg, standardMsg))

    # ported from 2.7's unittest
    def assertIsNotNone(self, obj, msg=None):
        """Included for symmetry with assertIsNone."""
        if obj is None:
            standardMsg = 'unexpectedly None'
            self.fail(self._formatMessage(msg, standardMsg))


def mock(class_or_module, orig_obj, mock_obj):
    """This is a decorator to be applied to any test method that needs to mock
    some module/class object (be it function, method or variable). It replaces
    the original object with any arbitrary object. The original is still
    accessible through "<original object name>_orig" attribute.

    Parameters:
    class_or_module - module/class in which the original function/method is
                      defined
    orig_obj - name of the original object as a string
    mock_obj - object that will replace the orig_obj, for example:
               -- instance of some fake function
               -- string

    Example:
    @tests.mock(utils, "run_subprocess", run_subprocess_mocked())
    -- replaces the original run_subprocess function from the utils module
       with the run_subprocess_mocked function.
    @tests.mock(logging.FileHandler, "_open", FileHandler_open_mocked())
    -- replaces the original _open method of the FileHandler class within
       the logging module with the FileHandler_open_mocked function.
    @tests.mock(gpgkey, "gpg_key_system_dir", "/nonexisting_dir/")
    -- replaces the gpgkey module-scoped variable gpg_key_system_dir with the
       "/nonexisting_dir/" string
    """
    def wrap(fn):
        # The @wraps decorator below makes sure the original object name
        # and docstring (in case of a method/function) are preserved.
        @wraps(fn)
        def wrapped_fn(*args, **kwargs):
            # Save temporarily the original object
            orig_obj_saved = getattr(class_or_module, orig_obj)
            # Replace the original object with the mocked one
            setattr(class_or_module, orig_obj, mock_obj)
            # To be able to use the original object within the mocked object
            # (e.g. to have the mocked function just as a wrapper for the
            # original function), save it as a temporary attribute
            # named "<original object name>_orig"
            orig_obj_attr = "{0}_orig".format(orig_obj)
            setattr(class_or_module, orig_obj_attr, orig_obj_saved)
            # Call the decorated test function
            return_value = fn(*args, **kwargs)
            # Restore the original object
            setattr(class_or_module, orig_obj, orig_obj_saved)
            # Remove the temporary attribute holding the original object
            delattr(class_or_module, orig_obj_attr)
            return return_value
        return wrapped_fn
    return wrap


class MockFunction(object):
    """This class should be used as a base class when creating a mocked
    function.

    Example:
    from convert2rhel import tests  # Imports tests/__init__.py
    class run_subprocess_mocked(tests.MockFunction):
        ...
    """
    __metaclass__ = abc.ABCMeta

    @abc.abstractmethod
    def __call__(self):
        """To be implemented when inheriting this class. The input parameters
        should either mimic the parameters of the original function/method OR
        use generic input parameters (*args, **kwargs).

        Examples:
        def __call__(self, cmd, print_output=True, shell=False):
            # ret_val to be set within the test or within __init__ first
            return self.ret_val

        def __call__(self, *args, **kwargs):
            pass
        """



from __future__ import unicode_literals
import sys
import os

prefix = "preupgrade"

defenc = "utf-8" if sys.getdefaultencoding() == "ascii" else sys.getdefaultencoding()

# dir where results of analysis are stored
assessment_results_dir = os.path.join("/root", prefix)

results_postfix = "-results"
# Dir where tar balls are placed
tarball_result_dir = assessment_results_dir + results_postfix

xccdf_template = "xccdf_template.xml"

# base name of XML and HTML file with results
result_prefix = "result"

tarball_base = result_prefix + 's'
tarball_prefix = "preupg_"
tarball_name = tarball_prefix + tarball_base + "-{0}"

xml_result_name = result_prefix + '.xml'
html_result_name = result_prefix + '.html'

xsl_sheet = "xccdf-report.xsl"

share_dir = "/usr/share"
# sources delivered by preupgrade assistant package
source_dir = os.path.join(share_dir, prefix)

data_dir = os.path.join(source_dir, "data")
# set data_dir value based on env variable PREUPG_DATA_DIR
# if variable is not set doesn't change the value
data_dir = os.getenv('PREUPG_DATA_DIR', data_dir)

# file where the lock file stored
lock_file = "/var/run/preupgrade.pid"

# dir where the cached logs are stored
cache_dir = "/var/cache/preupgrade"
log_dir = "/var/log/preupgrade"

# file with module set meta info
properties_ini = "properties.ini"
module_ini = "module.ini"

solution_txt = "solution.txt"
check_script = "check"
# preupg log file
preupg_log = os.path.join(log_dir, "preupg.log")

# preupg report log file
preupg_report_log = os.path.join(log_dir, "preupg-report.log")

# dir where the postupgrade scripts are placed
postupgrade_dir = "postupgrade.d"

# dir with scripts which are to be executed before reboot and upgrade.
preupgrade_scripts_dir = "preupgrade-scripts"
preupgrade_scripts_path = os.path.join(assessment_results_dir,
                                       preupgrade_scripts_dir)

# dirtyconfig directory used by preupgrade assistant
dirty_conf_dir = 'dirtyconf'

# cleanconfig directory used by preupgrade assistant
clean_conf_dir = 'cleanconf'

# cleanconf directory used by preupgrade assistant
# xccdf profile
profile = "xccdf_preupg_profile_default"

# name of dir with common files
common_name = "common"

# path to file with definitions of common scripts
common_scripts = os.path.join(data_dir, "preassessment", "scripts.txt")

# Default module set descriptor file
all_xccdf_xml_filename = "all-xccdf.xml"

# prefix of tag in xccdf files
xccdf_tag = "xccdf_preupg_rule_"

# name of the hash file
base_hashed_file = "hashed_file"

# name of the file which contains a list of rules
file_list_rules = "list_rules"

# kickstart directory name
kickstart_dir = "kickstart"

# kickstart postinstall directory name
postmigrate_dir = "postmigrate.d"

# kickstart and postupgrade.d directories
preupgrade_dirs = [dirty_conf_dir, clean_conf_dir,
                   'hooks', kickstart_dir, postupgrade_dir, 'common',
                   'preupgrade-scripts', 'noauto_postupgrade.d',
                   postmigrate_dir]

DOC_DIR = '/usr/share/doc/preupgrade-assistant/'
PREUPG_README = 'README'
readme_files = {PREUPG_README: PREUPG_README,
                'README.kickstart': os.path.join(kickstart_dir, 'README'),
                }

# Used for autogeneration check script issues
autocomplete = True

needs_inspection = "needs_inspection"
needs_action = "needs_action"

openscap_binary = "/usr/bin/oscap"

# The full license text
license = u"""Preupgrade Assistant performs system upgradability assessment
and gathers information required for successful operating system upgrade.
Copyright (C) 2013 Red Hat Inc.
%s
<new_line>
This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.
<new_line>
This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.
<new_line>
You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>."""

warning_text = "The Preupgrade Assistant is a diagnostics tool \n" \
               "and does not perform the actual upgrade."
migration_text = "The running system is 32bit. Migration is possible only to 64bit system.\n" \
                 "See help --dst-arch option.\n"
migration_options = ['x86_64', 'ppc64']
assessment_text = "Assessment of the system, running checks / SCE scripts"
result_text = "Result table with checks and their results for '{0}':"
risks_found_warning = "We have found some potential risks.\n" \
                      "Read the full report file '{0}' for more details."
upgrade_backup_warning = \
               "Please ensure you have backed up your system and/or data \n" \
               "before doing a system upgrade to prevent loss of data in \n" \
               "case the upgrade fails and full re-install of the system \n" \
               "from installation media is needed."
converter_message = "At least one of these converters ({0}) needs to be installed."
kickstart_text = "The Preupgrade Assistant generates a kickstart file in '%s'.\n" \
                 "The Kickstart file contains:\n" \
                 "- users with UID/GID which you should create on Red Hat Enterprise Linux 7 system.\n" \
                 "- the partitioning layout which was used on this system\n" \
                 "- the package set which was installed on this system.\n" \
                 "- the firewall rules which were enabled on this system.\n" \
                 "The Kickstart file is pre-generated from this system and is not to be used directly for \n" \
                 "the installation of Red Hat Enterprise Linux 7.\n" \
                 "The Kickstart file needs to be modified by the administrator.\n" \
                 "These directories exist on the migration system:\n" \
                 "- %s/cleanconf - configuration files which can be used on the migrated system.\n" \
                 "- %s/dirtyconf - configuration files which need to be clarified by the administrator.\n"

options_not_allowed = "Options --mode and --select-rules are not allowed together.\n"
unknown_rules = "These rules do not exist:\n%s\n"
text_converters = {'w3m': '{0} -T text/html -dump {1} > {2}',
                   'lynx': '{0} -nonumbers -nolist -force_html -dump -nolist -width=255 {1} > {2}',
                   'elinks': '{0} --no-references -dump-width 255 --no-numbering -dump {1} > {2}',
                   }

ui_command = "preupg -u http://example.com:8099/submit/ -r {0}"
openssl_command = "openssl x509 -text -in {0} | grep -A1 1.3.6.1.4.1.2312.9.1"

UPGRADE_PATH = ""
KS_DIR = os.path.join(assessment_results_dir, 'kickstart')
KS_FILENAME = 'preupgrade.ks'
KS_PATH = os.path.join(KS_DIR, KS_FILENAME)
KS_TEMPLATE = KS_FILENAME
KS_POSTSCRIPT_TEMPLATE = 'postmigrate.sh'
KS_TEMPLATES = [KS_TEMPLATE, KS_POSTSCRIPT_TEMPLATE]
KS_FILES = ['default_grouplist-el7', 'default-optional_grouplist-el7']
KS_SCRIPTS = "kickstart_scripts.txt"

PREUPG_CONFIG_FILE = os.path.join('/etc', 'preupgrade-assistant.conf')

DEVEL_MODE = os.path.join(cache_dir, 'devel_mode')

# Ordered dictionary because of python 2.4.

ORDERED_LIST = ['error', 'fail', 'needs_action', 'needs_inspection',
                'fixed', 'informational', 'not_applicable', 'not_selected',
                'not_checked', 'pass']


class ReturnValues(object):

    SCENARIO = 20
    MODE_SELECT_RULES = 21
    INVALID_CLI_OPTION = 22
    ROOT = 23
    PREUPG_BEFORE_RISKCHECK = 24
    MISSING_OPENSCAP = 25
    MISSING_TEXT_CONVERTOR = 26
    SCRIPT_TXT_MISSING = 27
    SEND_REPORT_TO_UI = 28
    INTERNAL_EXCEPTION = 29
    USER_ABORT = 30


class ModuleValues(object):

    ERROR = 2
    FAIL = 2
    NEEDS_ACTION = 1
    NEEDS_INSPECTION = 0
    FIXED = 0
    INFORMATIONAL = 0
    NOT_ALL = 0
    PASS = 0

PREUPG_RETURN_VALUES = {'error': ModuleValues.ERROR,
                        'fail': ModuleValues.FAIL,
                        'needs_action': ModuleValues.NEEDS_ACTION,
                        'needs_inspection': ModuleValues.NEEDS_INSPECTION,
                        'fixed': ModuleValues.FIXED,
                        'informational': ModuleValues.INFORMATIONAL,
                        'not_applicable': ModuleValues.NOT_ALL,
                        'not_selected': ModuleValues.NOT_ALL,
                        'not_checked': ModuleValues.NOT_ALL,
                        'pass': ModuleValues.PASS
                        }
ERROR_RETURN_VALUES = ['error', 'pass', 'informational', 'fixed',
                       'not_applicable', 'not_selected',
                       'not_checked' ]



# -*- coding: utf-8 -*-
from datetime import timedelta

LANG = "en"

# Singular and plural forms of time units in your language.
unit_names = dict(en = {"year" : ("year", "years"),
                        "month" : ("month", "months"),
                        "week" : ("week", "weeks"),
                        "day" : ("day", "days"),
                        "hour" : ("hour", "hours"),
                        "minute" : ("minute", "minutes"),
                        "second" : ("second", "seconds")})

num_repr = dict(en = {1 : "one",
                      2 : "two",
                      3 : "three",
                      4 : "four",
                      5 : "five",
                      6 : "six",
                      7 : "seven",
                      8 : "eight",
                      9 : "nine",
                      10 : "ten",
                      11 : "eleven",
                      12 : "twelve"})

def amount_to_str(amount):
    if amount in num_repr[LANG]:
        return num_repr[LANG][amount]
    return str(amount)

def seconds_in_units(seconds):
    """
    Returns a tuple containing the most appropriate unit for the
    number of seconds supplied and the value in that units form.

        >>> seconds_in_units(7700)
        (2, 'hour')
    """
    unit_limits = [("year", 365 * 24 * 3600),
                   ("month", 30 * 24 * 3600),
                   ("week", 7 * 24 * 3600),
                   ("day", 24 * 3600),
                   ("hour", 3600),
                   ("minute", 60),
                   ('second', 1)]
    result = []
    if seconds == 0:
        return [(0, 'second')]
    for unit_name, limit in unit_limits:
        if seconds >= limit:
            #amount = int(round(float(seconds) / limit))
            result.append((seconds / limit, unit_name))
            seconds = seconds % limit
    return result

def stringify(td):
    """
    Converts a timedelta into a nicely readable string.

        >>> td = timedelta(days = 77, seconds = 5)
        >>> print readable_timedelta(td)
        two months
    """
    seconds = td.days * 3600 * 24 + td.seconds
    stringified = seconds_in_units(seconds)

    # Localize it.
    processed = []
    for s in stringified:
        #i18n_amount = amount_to_str(amount, unit_name)
        amount, unit_name = s
        i18n_amount = str(amount)
        i18n_unit = unit_names[LANG][unit_name][1]
        if amount == 1:
            i18n_unit = unit_names[LANG][unit_name][0]
        processed.append("%s %s" % (i18n_amount, i18n_unit))
    return ' '.join(processed)


def test(td, s):
    try:
        assert stringify(td) == s
    except AssertionError:
        print 'ERROR', stringify(td), '!=', s


def main():
    global LANG
    LANG = "en"
    test(timedelta(weeks=7, days=3), '1 month 3 weeks 1 day')
    test(timedelta(weeks=1), '1 week')
    test(timedelta(days=1000), '2 years 9 months')
    test(timedelta(days=400), '1 year 1 month 5 days')
    test(timedelta(days=4), '4 days')
    test(timedelta(seconds=2000), '33 minutes 20 seconds')
    test(timedelta(seconds=9888), '2 hours 44 minutes 48 seconds')
    test(timedelta(seconds=99), '1 minute 39 seconds')
    test(timedelta(seconds=45), '45 seconds')
    test(timedelta(), '0 seconds')
    test(timedelta(seconds=1), '1 second')

if __name__ == "__main__":
    main()



from __future__ import unicode_literals
TAG_GROUP = "xccdf_preupg_group_"
TAG_VALUE = "xccdf_preupg_value_"
TAG_RULE = "xccdf_preupg_rule_"
GROUP_INI = """<xf:xccdf-fragment xmlns="http://checklists.nist.gov/xccdf/1.2"
 xmlns:xf="http://preupgrade-assistant.org/wiki/XCCDF-fragment"
  xmlns:xhtml="http://www.w3.org/1999/xhtml">
    <Group id=\""""+TAG_GROUP+"""{main_dir}" selected="true">
    <title>{group_title}</title>
    {group_value}
    </Group>
</xf:xccdf-fragment>
"""

CONTENT_INI = """<xf:xccdf-fragment xmlns="http://checklists.nist.gov/xccdf/1.2"
 xmlns:xf="http://preupgrade-assistant.org/wiki/XCCDF-fragment"
  xmlns:xhtml="http://www.w3.org/1999/xhtml">
    <Profile id="xccdf_preupg_profile_default">
        {select_rules}
    </Profile>
    <Group id=\""""+TAG_GROUP+"""{main_dir}" selected="true">
    <title>{group_title}</title>
    {group_value}
    {rule_tag}
    </Group>
</xf:xccdf-fragment>
"""
SELECT_TAG = """<select idref=\""""+TAG_RULE+"""{main_dir}_{scap_name}\" selected="true" />"""

VALUE = """
    <Value id=\""""+TAG_VALUE+"""{main_dir}_{scap_name}_state_{val}" operator="equals" type="string">
        <value>{value_name}</value>
    </Value>
"""
RULE_SECTION = """
    <Rule id=\""""+TAG_RULE+"""{main_dir}_{scap_name}" selected="true">
      <title>{content_title}</title>
      <description>
        {content_description}
        {check_description}
        {config_section}
      </description>
      <fixtext>{solution_text}</fixtext>
      <check system="http://open-scap.org/page/SCE">
        <check-import import-name="stdout" />
        {check_export}
        <check-content-ref href="{check_script}" />
      </check>
    </Rule>
"""
CONFIG_SECTION = """
        <xhtml:p>
            File(s) affected:
            <xhtml:ul>
            {config_file}
            </xhtml:ul>
        </xhtml:p>
"""
RULE_SECTION_VALUE_IMPORT = """\t\t<check-import import-name="stderr"/>"""

RULE_SECTION_VALUE = """\t\t<check-export export-name="{value_name_upper}" value-id=\""""+TAG_VALUE+"""{main_dir}_{scap_name}_state_{val}" />
"""
DIC_VALUES = {'current_directory': '/root/preupgrade',
              'module_path': '',
              }

GLOBAL_DIC_VALUES = {'tmp_preupgrade': 'SCENARIO',
                     'migrate': '0',
                     'upgrade': '0',
                     'report_dir': '/root/preupgrade',
                     'devel_mode': '0',
                     'dist_native': ''
                     }

RULE_SECTION_VALUE_GLOBAL = """\t\t<check-export export-name="{value_name_upper}" value-id=\""""+TAG_VALUE+"""{value_name}" />
"""



from __future__ import unicode_literals
import unittest
import tempfile
import shutil
import os

from preupg.application import Application
from preupg.conf import Conf, DummyConf
from preupg.cli import CLI
from preupg import settings, xml_manager
from preupg.utils import (PostupgradeHelper, FileHelper,
                          OpenSCAPHelper, ModuleSetUtils)
from preupg.report_parser import ReportParser

try:
    import base
except ImportError:
    import tests.base as base


def setup_preupg_environment(args, content, tmp_dir, mode=None):
    conf = {
        "contents": content,
        "profile": "xccdf_preupg_profile_default",
        "assessment_results_dir": tmp_dir,
        "skip_common": True,
        "temp_dir": tmp_dir,
        "id": None,
        "debug": True,  # so root check won't fail
        "mode": mode,
    }
    dc = DummyConf(**conf)
    cli = CLI(args)
    a = Application(Conf(dc, settings, cli))
    # Prepare all variables for test
    a.conf.source_dir = os.getcwd()
    a.determine_module_set_location()
    a.openscap_helper = OpenSCAPHelper(a.conf.assessment_results_dir,
                                       a.conf.result_prefix,
                                       a.conf.xml_result_name,
                                       a.conf.html_result_name,
                                       a.all_xccdf_xml_path)
    return a


class TestPreupg(base.TestCase):
    temp_dir = None

    def setUp(self):
        self.temp_dir = tempfile.mkdtemp()

    def tearDown(self):
        shutil.rmtree(self.temp_dir)

    def test_all(self):

        """Basic test for whole program"""

        content = "tests/generated_results/all-xccdf-migrate.xml"
        args = ["--contents", content]
        a = setup_preupg_environment(args, content, self.temp_dir)
        self.assertEqual(a.run_scan(), 0)


class TestPreupgMigrate(base.TestCase):
    temp_dir = None

    def setUp(self):
        self.temp_dir = tempfile.mkdtemp()

    def tearDown(self):
        shutil.rmtree(self.temp_dir)

    def test_migrate(self):
        """Basic test for whole program"""

        content = "tests/generated_results/all-xccdf-migrate.xml"
        args = ["--contents", content, "--mode", "migrate"]
        a = setup_preupg_environment(args, content, self.temp_dir, mode='migrate')
        self.assertEqual(a.run_scan(), 0)
        rp = ReportParser(os.path.join(self.temp_dir,
                                       settings.xml_result_name))
        rp.modify_result_path(self.temp_dir, "FOOBAR6_7", 'migrate')
        for values in rp.get_nodes(rp.target_tree, "Value", ".//"):
            self.assertTrue(values.get("id"))
            if values.get("id").endswith("_state_migrate"):
                for value in rp.get_nodes(values, "value"):
                    self.assertEqual(int(value.text), 1)
            if values.get("id").endswith("_state_upgrade"):
                for value in rp.get_nodes(values, "value"):
                    self.assertEqual(int(value.text), 0)


class TestPreupgUpgrade(base.TestCase):
    temp_dir = None

    def setUp(self):
        self.temp_dir = tempfile.mkdtemp()

    def tearDown(self):
        shutil.rmtree(self.temp_dir)

    def test_upgrade(self):
        """Basic test for whole program"""

        content = "tests/generated_results/all-xccdf-upgrade.xml"
        args = ["--contents", content, "--mode", "upgrade"]
        a = setup_preupg_environment(args, content, self.temp_dir, mode='upgrade')
        self.assertEqual(a.run_scan(), 0)
        rp = ReportParser(os.path.join(self.temp_dir, "result.xml"))
        rp.modify_result_path(self.temp_dir, "FOOBAR6_7", 'upgrade')
        for values in rp.get_nodes(rp.target_tree, "Value", ".//"):
            if values.get("id").endswith("_state_migrate"):
                for value in rp.get_nodes(values, "value"):
                    self.assertEqual(int(value.text), 0)
            if values.get("id").endswith("_state_upgrade"):
                for value in rp.get_nodes(values, "value"):
                    self.assertEqual(int(value.text), 1)


class TestXMLUpdates(base.TestCase):
    content = None
    test_content = None

    def setUp(self):
        self.content = "tests/generated_results/all-xccdf-upgrade.xml"
        self.test_content = self.content+".test"

    def tearDown(self):
        os.remove(self.test_content)

    def test_result_dirs_tmp_preupgrade(self):
        shutil.copyfile(self.content, self.test_content)
        rp = ReportParser(self.test_content)
        result_path = "/abc/def"
        rp.modify_result_path(result_path, "FOOBAR6_7", 'migrate')
        found_tmp = 0

        for values in rp.get_nodes(rp.target_tree, "Value", prefix='./'):
            if values.get("id").endswith("_tmp_preupgrade"):
                for value in rp.get_nodes(values, "value"):
                    if value.text == result_path:
                        found_tmp = 1

        self.assertEquals(found_tmp, 1)

    def test_result_dirs_current_dir(self):
        shutil.copyfile(self.content, self.test_content)
        rp = ReportParser(self.test_content)
        result_path = "/abc/efg"
        scenario = 'FOOBAR6_7'
        rp.modify_result_path(result_path, scenario, 'migrate')
        found_current = 0
        for values in rp.get_nodes(rp.target_tree, "Value", ".//"):
            if values.get("id").endswith("_preupg_state_current_directory"):
                for value in rp.get_nodes(values, "value"):
                    result_dir = result_path+"/"+scenario+"/dummy_preupg"
                    if value.text == result_dir:
                        found_current = 1

        self.assertEquals(found_current, 1)


class TestCLI(base.TestCase):
    def test_opts(self):
        """ basic test of several options """
        conf = {
            "scan": "FOOBAR6_7",
            "skip_common": False,
            "id": 1,
            "list_contents_set": True,
            "verbose": 1,
            "text": True,
            "contents": "content/FOOBAR6_7",
            "cleanup": True,
            "mode": 'upgrade',
            "select_rules": "abc",
            "list_rules": True,
            "version": True,
            "force": True,
            "riskcheck": True,
        }
        dc = DummyConf(**conf)
        cli = CLI(["--scan", "FOOBAR6_7", "--skip-common", "--cleanup",
                   "--list-contents-set", "--verbose", "--text", "--force",
                   "--mode", "upgrade", "--select-rules", "abc", "--riskcheck",
                   "--list-rules"])
        a = Application(Conf(cli.opts, dc, cli))

        self.assertTrue(a.conf.skip_common)
        self.assertEqual(a.conf.contents, "content/FOOBAR6_7")
        self.assertTrue(a.conf.list_contents_set)
        self.assertTrue(a.conf.list_rules)
        self.assertTrue(a.conf.force)
        self.assertTrue(a.conf.text)
        self.assertTrue(a.conf.cleanup)
        self.assertEqual(int(a.conf.verbose), 1)
        self.assertEqual(a.conf.scan, "FOOBAR6_7")
        self.assertEqual(a.conf.mode, "upgrade")
        self.assertEqual(a.conf.select_rules, "abc")
        self.assertTrue(a.conf.riskcheck)


class TestHashes(base.TestCase):
    dir_name = None

    def setUp(self):
        self.dir_name = tempfile.mkdtemp()

    def tearDown(self):
        shutil.rmtree(self.dir_name)

    def test_hashes(self):
        text_to_hash="""
            This is preupgrade assistant test has string"
        """
        self.dir_name = "tests/hashes"
        os.mkdir(self.dir_name)
        FileHelper.write_to_file(os.path.join(self.dir_name, "post_script"), 'wb', text_to_hash)
        PostupgradeHelper.hash_postupgrade_file(False, self.dir_name)
        return_value = PostupgradeHelper.hash_postupgrade_file(False, self.dir_name, check=True)
        self.assertTrue(return_value)


class TestSolutionReplacement(base.TestCase):

    def test_solution_bold_tag(self):
        solution_text = 'This is solution text [bold: provided as text ] by check script'
        expected_text = 'This is solution text <b> provided as text </b> by check script'
        line = xml_manager.tag_formating(solution_text)
        self.assertEqual(expected_text, line)

    def test_solution_www_link_tag(self):
        solution_text = 'This is solution text [link: http://127.0.0.1/all-xccdf.html ] by check script'
        expected_text = 'This is solution text <a href="http://127.0.0.1/all-xccdf.html">http://127.0.0.1/all-xccdf.html</a> by check script'
        line = xml_manager.tag_formating(solution_text)
        self.assertEqual(expected_text, line)

    def test_solution_file_link_tag(self):
        solution_text = 'This is solution text [link: description.txt ] by check script'
        expected_text = 'This is solution text <a href="./description.txt">description.txt</a> by check script'
        line = xml_manager.tag_formating(solution_text)
        self.assertEqual(expected_text, line)

    def test_solution_wrong_link_tag(self):
        solution_text = 'This is solution text [link: /var/cache/description.txt] by check script'
        expected_text = 'This is solution text /var/cache/description.txt by check script'
        line = xml_manager.tag_formating(solution_text)
        self.assertEqual(expected_text, line)

    def test_solution_file_bold_tag(self):
        solution_text = 'This is solution text [link: description.txt ] by [bold: check script ]'
        expected_text = 'This is solution text <a href="./description.txt">description.txt</a> by <b> check script </b>'
        line = xml_manager.tag_formating(solution_text)
        self.assertEqual(expected_text, line)

    def test_solution_www_bold_tag(self):
        solution_text = 'This is solution text [link: http://127.0.0.1/description.txt ] by [bold: check script ]'
        expected_text = 'This is solution text <a href="http://127.0.0.1/description.txt">http://127.0.0.1/description.txt</a> by <b> check script </b>'
        line = xml_manager.tag_formating(solution_text)
        self.assertEqual(expected_text, line)


class TestModuleSet(base.TestCase):
    '''
    Test get_scenario method, to get directory with modules
    '''
    def test_correct_module_set(self):
        '''
        Test to get right module directory with contents option,
        directory with modules are the same where all-xccdf file is located
        '''
        conf = {
            "contents": "tests/Modules/all-xccdf.xml",
        }
        dummy_conf = DummyConf(**conf)
        cli = CLI(["--contents", "tests/Modules/all-xccdf.xml"])
        app = Application(Conf(dummy_conf, settings, cli))
        # Prepare all variables for test
        app.conf.source_dir = os.getcwd()
        app.content = app.conf.contents
        app.basename = os.path.basename(app.content)
        app.determine_module_set_location()
        self.assertEqual(app.module_set_dirname, 'Modules')


class TestModuleSetConfigParse(base.TestCase):
    '''
    Test parser of source and destination major versions of system from module
    set config file
    '''
    def test_valid_config(self):
        '''
        Test check if parse on correct module set config works fine
        '''
        this_file_dir_path = os.path.dirname(os.path.realpath(__file__))
        dummy_config_path = os.path.join(this_file_dir_path, 'FOOBAR6_7')
        version = ModuleSetUtils.get_module_set_os_versions(dummy_config_path)
        self.assertEqual(version, ['6', '7'])

    def test_invalid_config(self):
        '''
        Test with wrong module set config path
        '''
        self.assertRaises(EnvironmentError,
                          ModuleSetUtils.get_module_set_os_versions,
                          '/dev/null')


class TestModuleSetConfigContent(base.TestCase):
    '''
    Test case for validation keys and sections inside module set config file
    '''
    def __init__(self, *args, **kwargs):
        super(TestModuleSetConfigContent, self).__init__(*args, **kwargs)
        self.src_version_key = "src_major_version"
        self.dst_version_key = "dst_major_version"
        self.section_name = "preupgrade-assistant-modules"
        self.this_file_dir_path = os.path.dirname(os.path.realpath(__file__))
        self.dummy_config_path = os.path.join(self.this_file_dir_path,
                                              'FOOBAR6_7/properties.ini')

    def test_valid_keys(self):
        '''
        Test to check correct valid keys in module set config file
        '''
        try:
            for key in [self.src_version_key, self.dst_version_key]:
                ModuleSetUtils.get_config_key_value(self.dummy_config_path,
                                                    key,
                                                    self.section_name)
        except EnvironmentError:
            self.fail('Key: {0} in {1} file should be correct'.format(
                self.src_version_key, self.dummy_config_path))

    def test_invalid_key(self):
        '''
        Test to check some invalid non existing key in module set config file
        '''
        self.assertRaises(EnvironmentError,
                          ModuleSetUtils.get_config_key_value,
                          self.dummy_config_path, "invalid_key",
                          self.section_name)

    def test_invalid_section(self):
        '''
        Test to check some invalid non existing section in module set config
        file
        '''
        self.assertRaises(EnvironmentError,
                          ModuleSetUtils.get_config_key_value,
                          self.dummy_config_path, self.src_version_key,
                          "invalid_section")


def suite():
    loader = unittest.TestLoader()
    suite = unittest.TestSuite()
    suite.addTest(loader.loadTestsFromTestCase(TestPreupg))
    suite.addTest(loader.loadTestsFromTestCase(TestPreupgMigrate))
    suite.addTest(loader.loadTestsFromTestCase(TestPreupgUpgrade))
    suite.addTest(loader.loadTestsFromTestCase(TestCLI))
    suite.addTest(loader.loadTestsFromTestCase(TestHashes))
    suite.addTest(loader.loadTestsFromTestCase(TestSolutionReplacement))
    suite.addTest(loader.loadTestsFromTestCase(TestXMLUpdates))
    suite.addTest(loader.loadTestsFromTestCase(TestModuleSet))
    suite.addTest(loader.loadTestsFromTestCase(TestModuleSetConfigParse))
    suite.addTest(loader.loadTestsFromTestCase(TestModuleSetConfigContent))
    return suite


if __name__ == '__main__':
    unittest.TextTestRunner(verbosity=3).run(suite())



# -*- coding: utf-8 -*-
"""
The application module serves for running oscap binary and reporting results
to UI.
"""

from __future__ import unicode_literals, print_function
import shutil
import datetime
import os
import subprocess
import sys
import logging
from distutils import dir_util

try:
    from xmlrpclib import Fault
except ImportError:
    from xmlrpc.client import Fault

from preupg import xml_manager, settings, exception
from preupg.common import Common
from preupg.settings import ReturnValues
from preupg.scanning import ScanProgress, ScanningHelper
from preupg.utils import (FileHelper, ProcessHelper, DirHelper, OpenSCAPHelper,
                          MessageHelper, TarballHelper, SystemIdentification,
                          PostupgradeHelper, ConfigHelper, ConfigFilesHelper,
                          ModuleSetUtils)
from preupg.xccdf import XccdfHelper
from preupg.logger import log_message, LoggerHelper, logger, logger_report
from preupg.logger import logger_debug
from preupg.report_parser import ReportParser
from preupg.kickstart.application import KickstartGenerator
from preupg.xmlgen.compose import XCCDFCompose
from preupg.version import VERSION


def fault_repr(self):
    """Monkey patching Fault's repr method to interpret newlines."""
    log_message(self.faultString)
    return "<Fault %s: %s>" % (self.faultCode, self.faultString)


Fault.__repr__ = fault_repr


def get_installed_module_sets(source_dir):
    """Return a dictionary of installed module sets within a directory. Format
    of the dictionary:
      {<module set root dir name>: <all xccdf xml full path>}
    """
    module_sets = {}
    is_dir = lambda x: os.path.isdir(os.path.join(source_dir, x))
    dirs = os.listdir(source_dir)
    for module_set_root_dir in filter(is_dir, dirs):
        all_xccdf_xml_path = os.path.join(
            source_dir, module_set_root_dir, settings.all_xccdf_xml_filename)
        if os.path.exists(all_xccdf_xml_path):
            logger_debug.info('%s', module_set_root_dir)
            module_sets[module_set_root_dir] = all_xccdf_xml_path
    return module_sets


def show_message(message):
    """
    Prints message out on stdout message (kind of yes/no) and return answer.

    Return values are:
    Return True on accept (y/yes). Otherwise returns False
    """
    accept = ['y', 'yes']
    choice = MessageHelper.get_message(title=message, prompt='[Y/n]')
    if choice.lower() in accept:
        return True
    else:
        return False


class Application(object):

    """Class for oscap binary and reporting results to UI"""

    def __init__(self, conf):
        """conf is preupg.conf.Conf object, contains configuration"""
        self.conf = conf
        self.all_xccdf_xml_path = ""
        self.all_xccdf_xml_copy_path = ""
        self.module_set_dirname = ""
        self.module_set_path = ""
        self.module_set_copy_path = ""
        self.result_file = ""
        self.xml_mgr = None
        self.scanning_progress = None
        self.report_parser = None
        self.text_convertor = ""
        self.common = None
        self._devel_mode = 0
        self._dist_mode = None
        self.report_log_file = None
        self.debug_log_file = None
        settings.profile = self.conf.profile
        if self.conf.debug is None:
            LoggerHelper.add_stream_handler(logger, logging.INFO)
        else:
            LoggerHelper.add_stream_handler(logger, logging.DEBUG)
        self.openscap_helper = None
        self._add_report_log_file()
        self._add_debug_log_file()
        self.tar_ball_name = None
        self._set_old_report_style()

    def _add_report_log_file(self):
        """
        Add the special report log file
        :return:
        """
        try:
            LoggerHelper.add_file_handler(
                logger_report, settings.preupg_report_log,
                formatter=logging.Formatter(
                    "%(asctime)s %(filename)s:%(lineno)s %(funcName)s:"
                    " %(message)s"),
                level=logging.DEBUG
            )
        except (IOError, OSError):
            logger.warning("Can not create report log '%s'",
                           settings.preupg_report_log)
        else:
            self.report_log_file = settings.preupg_report_log

    def _add_debug_log_file(self):
        """
        Add the special report log file
        :return:
        """
        try:
            LoggerHelper.add_file_handler(
                logger_debug, settings.preupg_log,
                formatter=logging.Formatter(
                    "%(asctime)s %(levelname)s\t%(filename)s"
                    ":%(lineno)s %(funcName)s: %(message)s"),
                level=logging.DEBUG
            )
        except (IOError, OSError):
            logger.warning("Can not create debug log '%s'",
                           settings.preupg_log)
        else:
            self.debug_log_file = settings.preupg_log

    def _set_old_report_style(self):
        """
        Choose which HTML report style should be used.

        Set self.old_report_style to True when it is required from commandline
        or older version of OpenSCAP has been detected.
        """
        if self.conf.old_report_style:
            self.old_report_style = True
        elif OpenSCAPHelper.is_oscap_equal_or_greater(1, 2, 7) is False:
            # in case that OpenSCAP version is lower then 1.2.7, fallback
            # to the old (simple) report style.
            log_message("Generating simply styled report due to the "
                        "limitations of the installed OpenSCAP")
            self.old_report_style = True
        else:
            self.old_report_style = False

    def get_postupgrade_dir(self):
        """Function returns postupgrade dir"""
        return os.path.join(self.conf.assessment_results_dir,
                            settings.postupgrade_dir)

    def upload_results(self):
        """upload tarball with results to frontend"""
        import xmlrpclib
        import socket
        url = ""
        if self.conf.upload is True:
            # lets try default configuration
            log_message('Specify the server where to upload the results.')
            log_message(settings.ui_command.format(self.conf.results))
            return False
        else:
            if self.conf.upload[-1] == '/':
                url = self.conf.upload
            else:
                url = self.conf.upload + '/'
        try:
            proxy = xmlrpclib.ServerProxy(url)
            proxy.submit.ping()
        except Exception as ex:
            message = 'Can\'t connect to preupgrade assistant WEB-UI at %s.' \
                      '\n\nPlease ensure that package' \
                      ' preupgrade-assistant-ui has been installed on target' \
                      ' system and firewall is set up ' \
                      'to allow connections on port 8099.' % url
            log_message(message)
            log_message(ex.__str__())
            return False

        if not self.conf.results:
            tarball_results = TarballHelper.get_latest_tarball(
                settings.tarball_result_dir)
        else:
            tarball_results = self.conf.results
        if tarball_results is None or not os.path.exists(tarball_results):
            log_message("Can't determine what tarball to upload to the UI.",
                        level=logging.ERROR)
            return False
        file_content = FileHelper.get_file_content(tarball_results, 'rb',
                                                   False, False)

        binary = xmlrpclib.Binary(file_content)
        host = socket.gethostname()
        response = proxy.submit.submit_new({
            'data': binary,
            'host': host,
        })
        try:
            status = response['status']
        except KeyError:
            log_message('Invalid response from the server.')
            log_message("Invalid response from the server: %s"
                        % response, level=logging.ERROR)
            return False
        else:
            if status == 'OK':
                try:
                    url = response['url']
                except KeyError:
                    log_message('The report submitted successfully.')
                else:
                    log_message('The report submitted successfully. You can inspect it at %s.' % url)
            else:
                try:
                    message = response['message']
                    log_message('The report not submitted. The server returned a message: ', message)
                    log_message("The report status: %s (%s)" % (status, message), level=logging.ERROR)
                except KeyError:
                    log_message('The report not submitted. The server returned a status: ', status)
                    log_message("The report status: %s" % status, level=logging.ERROR)
                return False
        return True

    def prepare_scan_directories(self):
        """Used for prepartion of directories used during scan functionality"""
        dirs = [self.conf.assessment_results_dir, settings.tarball_result_dir]
        dirs.extend(os.path.join(self.conf.assessment_results_dir, x) for x in settings.preupgrade_dirs)
        if self.conf.temp_dir:
            dirs.append(self.conf.temp_dir)
        for dir_name in dirs:
            DirHelper.check_or_create_temp_dir(dir_name)

        # Copy README files into assessment result directory so the user has
        # them easily available
        for orig_filename, dest_filename in settings.readme_files.items():
            shutil.copyfile(os.path.join(settings.DOC_DIR, orig_filename),
                            os.path.join(settings.assessment_results_dir,
                                         dest_filename))

    def get_total_check(self):
        """Returns a total check"""
        return self.report_parser.get_number_checks()

    def run_scan_process(self):
        """Function scans the source system"""
        self.xml_mgr = xml_manager.XmlManager(self.conf.assessment_results_dir,
                                              self.module_set_copy_path
                                              )

        self.report_parser.add_global_tags(self.conf.assessment_results_dir,
                                           self.rename_custom_module_set(
                                               self.module_set_dirname),
                                           self.conf.mode,
                                           self._devel_mode,
                                           self._dist_mode)

        self.report_parser.modify_result_path(self.conf.assessment_results_dir,
                                              self.rename_custom_module_set(
                                                  self.module_set_dirname),
                                              self.conf.mode)
        # Execute assessment
        self.scanning_progress = ScanProgress(self.get_total_check(), self.conf.debug)
        self.scanning_progress.set_names(self.report_parser.get_name_of_checks())
        log_message('%s:' % settings.assessment_text, new_line=True)
        log_message('%.3d/%.3d ...running (%s)' % (
                    1,
                    self.get_total_check(),
                    self.scanning_progress.get_full_name(0)),
                    new_line=False
                    )
        start_time = datetime.datetime.now()
        self.scanning_progress.time = start_time
        self.run_scan(function=self.scanning_progress.show_progress)
        end_time = datetime.datetime.now()
        diff = end_time - start_time
        log_message(
            "The assessment finished (time %.2d:%.2ds)" % (diff.seconds / 60,
                                                           diff.seconds % 60)
        )

    def run_scan(self, function=None):
        """
        The function is used for either scanning system or
        for applying changes on the target system
        """
        cmd = self.openscap_helper.build_command()
        logger_debug.debug('running_command: %s', cmd)
        return ProcessHelper.run_subprocess(cmd, print_output=False, function=function)

    def clean_preupgrade_environment(self):
        """
        Function cleans files created by preupgrade-assistant

        :return:
        """
        force_directories = [self.conf.assessment_results_dir]
        delete_directories = [settings.tarball_result_dir,
                              settings.cache_dir,
                              settings.log_dir]
        for dir_name in force_directories:
            if os.path.isdir(dir_name):
                shutil.rmtree(dir_name)
        for dir_name in delete_directories:
            for root, dirs, files in os.walk(dir_name, topdown=False):
                for name in files:
                    os.remove(os.path.join(root, name))
                for name in dirs:
                    shutil.rmtree(os.path.join(root, name))

    def clean_scan(self):
        """
        The function remove symlink /root/preupgrade from older versions
        Also it removes directory /root/preupgrade because of new assessment.
        """
        if os.path.islink(self.conf.assessment_results_dir):
            os.unlink(self.conf.assessment_results_dir)
        if os.path.isdir(self.conf.assessment_results_dir):
            shutil.rmtree(self.conf.assessment_results_dir)

    def prepare_xml_for_html(self):
        """The function prepares a XML file for HTML creation"""
        # Reload XML file
        self.report_parser.reload_xml(self.openscap_helper.get_default_xml_result_path())
        # strip whitespaces on start and end of stdout/stderr from modules
        # inside the result.xml file
        self.report_parser.strip_whitespaces()
        # Replace fail in case of slight and medium risks with needs_inspection
        self.report_parser.replace_inplace_risk(scanning_results=self.scanning_progress)
        if not self.conf.debug:
            self.report_parser.remove_debug_info()
        self.report_parser.reload_xml(self.openscap_helper.get_default_xml_result_path())
        self.report_parser.update_check_description()
        xml_report = self.openscap_helper.get_default_xml_result_path()
        if self.old_report_style:
            ReportParser.write_xccdf_version(xml_report, direction=True)

    def generate_html_or_text(self):
        self.generate_html()
        if self.conf.text:
            ProcessHelper.run_subprocess(self.get_cmd_convertor(), print_output=False, shell=True)

    def generate_html(self):
        """Convert XML to HTML"""
        xml_report = self.openscap_helper.get_default_xml_result_path()
        html_report = self.openscap_helper.get_default_html_result_path()
        self.openscap_helper.run_generate(xml_report,
                                          html_report,
                                          old_style=self.old_report_style)
        self.xml_mgr.update_report(html_report)

    def update_xml_after_html_generated(self):
        xml_report = self.openscap_helper.get_default_xml_result_path()
        self.xml_mgr.update_report(xml_report)
        if self.old_report_style:
            # Revert change to the XML XCCDF namespace which would break preupg-diff
            ReportParser.write_xccdf_version(xml_report)

    def copy_postupgrade_files(self):
        """
        Function copies postupgrade scripts and creates hash postupgrade file.
        """
        # Copy postupgrade.d special files
        PostupgradeHelper.special_postupgrade_scripts(self.conf.assessment_results_dir)
        PostupgradeHelper.hash_postupgrade_file(self.conf.verbose, self.get_postupgrade_dir())

    def get_cmd_convertor(self):
        """Function returns cmd with text convertor string"""
        cmd = settings.text_converters[self.text_convertor].format(
            self.text_convertor,
            self.openscap_helper.get_default_html_result_path(),
            self.openscap_helper.get_default_txt_result_path()
        )
        return cmd

    def rename_custom_module_set(self, module_set_dirname):
        if self.conf.contents:
            module_set_dirname = module_set_dirname.replace('-results', '')
        return module_set_dirname

    def prepare_scan_system(self):
        """
        Prepare system for scan.

        In case the result of previous assessment is detected, remove it
        and prepare new directory structure that will be used for next scan.
        Additionaly process scripts generating common data about the current
        system, generate final XCCDF compose from the original set of modules
        identified for run and in the end process initial script of modules to
        complete expected environment for the scan.
        """
        # First of all we need to delete the older one assessment
        self.clean_scan()
        self.prepare_scan_directories()
        self.common = Common(self.conf)
        if not self.conf.skip_common:
            if not self.common.common_results():
                return ReturnValues.SCRIPT_TXT_MISSING

        # Generate final XCCDF compose under self.module_set_copy_path
        xccdf_compose = XCCDFCompose(
            self.module_set_path, self.module_set_copy_path)
        ret_val = xccdf_compose.generate_xml()
        if ret_val != 0:
            return ret_val
        self.run_init()
        return 0

    def run_init(self):
        """
        Run module set's init script if exists

        If module set provides executable called 'init', run it.   Standard
        output is ignored.  If either exit status is non-zero or error is
        printed, raise ModuleSetInitError.

        Note that exported environment variables need to be reviewed; current
        set is purely to make previous solution work and prevent regression.
        """
        scenario = self.rename_custom_module_set(self.module_set_dirname)
        init = os.path.join(self.conf.source_dir, scenario, 'init')
        if os.access(init, os.F_OK):
            init_vars = {
                'PREUPGM_INIT_ASSESSMENT_DIR': self.module_set_copy_path,
                'PREUPGM_INIT_SCENARIO': scenario,
                'PREUPGM_INIT_DST_ARCH': self.conf.dst_arch if self.conf.dst_arch else "",
                'PREUPGM_INIT_CONTENTS': self.conf.contents if self.conf.contents else "",
            }
            logger_debug.debug("calling module init script with: %r" % init_vars)
            init_env = os.environ.copy()
            init_env.update(init_vars)
            try:
                p = subprocess.Popen(
                    [init],
                    env=init_env,
                    stdout=subprocess.PIPE,
                    stderr=subprocess.PIPE
                )
                out, err = p.communicate()
            except OSError as e:
                raise exception.ModuleSetFormatError("init script failed to execute", init, e)
            if p.returncode or err.strip():
                raise exception.ModuleSetInitError(p.returncode, err)

    @staticmethod
    def copy_preupgrade_scripts(assessment_dir):
        # Copy preupgrade-scripts directory from scenarvirtuio
        preupg_scripts = os.path.join(assessment_dir,
                                      settings.preupgrade_scripts_dir)
        if os.path.exists(preupg_scripts):
            dir_util.copy_tree(preupg_scripts,
                               settings.preupgrade_scripts_path)

    def scan_system(self):
        """The function is used for scanning system with all steps."""
        self._set_devel_mode()
        if not self.is_module_set_valid():
            return ReturnValues.SCENARIO
        ret_val = self.prepare_scan_system()
        if ret_val != 0:
            return ret_val
        # Update source XML file in temporary directory
        self.openscap_helper.update_variables(self.conf.assessment_results_dir,
                                              self.conf.result_prefix,
                                              self.conf.xml_result_name,
                                              self.conf.html_result_name,
                                              self.all_xccdf_xml_copy_path)
        try:
            self.report_parser = ReportParser(self.all_xccdf_xml_copy_path)
        except IOError:
            log_message("Error: Unable to open {0}."
                        .format(self.all_xccdf_xml_copy_path))
            return ReturnValues.SCENARIO
        if self.conf.mode:
            lines = [i.rstrip() for i in
                     FileHelper.get_file_content(
                         os.path.join(self.module_set_copy_path,
                                      self.conf.mode),
                         'rb', method=True)]
            self.report_parser.select_rules(lines)
        if self.conf.select_rules:
            lines = [i.strip() for i in self.conf.select_rules.split(',')]
            unknown_rules = self.report_parser.check_rules(lines)
            if unknown_rules:
                log_message(settings.unknown_rules % '\n'.join(unknown_rules))
            self.report_parser.select_rules(lines)
        self.run_scan_process()
        main_report = self.scanning_progress.get_output_data()
        self.prepare_xml_for_html()
        self.generate_html_or_text()
        self.update_xml_after_html_generated()
        self.copy_postupgrade_files()
        self.copy_preupgrade_scripts(self.module_set_copy_path)
        ConfigFilesHelper.copy_modified_config_files(
            settings.assessment_results_dir)

        # It prints out result in table format
        ScanningHelper.format_rules_to_table(main_report, "main contents")

        self.tar_ball_name = TarballHelper.tarball_result_dir(self.conf.tarball_name, self.conf.verbose)
        log_message("The tarball with results is stored in '%s' ." % self.tar_ball_name)
        log_message("The latest assessment is stored in the '%s' directory." % self.conf.assessment_results_dir)
        # pack all configuration files to tarball
        return 0

    def is_module_set_valid(self):
        if self.module_set_dirname is None:
            log_message('Invalid scenario: %s' % self.module_set_path)
            return False
        if not os.path.isdir(self.module_set_path):
            log_message('Invalid scenario: %s' % self.module_set_path,
                        level=logging.ERROR)
            return False
        # Validate properties.ini file in module set dir
        try:
            ModuleSetUtils.get_module_set_os_versions(self.module_set_path)
        except EnvironmentError as err:
            log_message(str(err), level=logging.ERROR)
            return False
        return True

    def summary_report(self, tarball_path):
        """Function prints a summary report"""
        command = settings.ui_command.format(tarball_path)
        if self.conf.text:
            path = self.openscap_helper.get_default_txt_result_path()
        else:
            path = self.openscap_helper.get_default_html_result_path()

        report_dict = {
            0: settings.risks_found_warning.format(path),
            1: settings.risks_found_warning.format(path),
            2: 'We have found some critical issues. In-place upgrade or migration is not advised.\n' +
               "Read the file {0} for more details.".format(path),
            3: 'We have found some error issues. In-place upgrade or migration is not advised.\n' +
               "Read the file {0} for more details.".format(path)
        }
        report_return_value = XccdfHelper.check_inplace_risk(
            self.openscap_helper.get_default_xml_result_path(),
            0)
        try:
            if report_dict[int(report_return_value)]:
                log_message('Summary information:')
                log_message(report_dict[int(report_return_value)])
        except KeyError:
            # We do not want to print anything in case of testing contents
            pass
        if not self.conf.mode or self.conf.mode == "upgrade":
            # User either has not specied mode (upgrade and migration both
            # together by default) or has chosen upgrade only = print warning
            # to backup the system before doing the in-place upgrade
            log_message(settings.upgrade_backup_warning)
        log_message("Upload results to UI by the command:\ne.g. {0} .".format(command))
        return report_return_value

    def _set_devel_mode(self):
        # Check for devel_mode
        if os.path.exists(settings.DEVEL_MODE):
            self._devel_mode = 1
            self._dist_mode = ConfigHelper.get_preupg_config_file(
                settings.PREUPG_CONFIG_FILE, 'dist_mode', section="devel-mode")
        else:
            self._devel_mode = 0

    def get_default_module_set_dirname(self):
        available_module_set_dirs = get_installed_module_sets(
            self.conf.source_dir)
        if not available_module_set_dirs:
            log_message("No modules found in the default directory (%s).\n"
                        " Either install a package with modules or use"
                        " -c option for custom created modules."
                        % settings.source_dir)
            return None
        if len(available_module_set_dirs) > 1:
            log_message("More than one module set is detected in the default"
                        " directory (%s)." % settings.source_dir)
            log_message("Available module sets: \n%s"
                        % '\n'.join(available_module_set_dirs))
            log_message("Use option -s to specify which module set should be"
                        " used.")
            return None
        return available_module_set_dirs.keys()[0]

    def determine_module_set_location(self):
        """Get module set path, directory name and path to the all_xccdf.xml"""
        if not self.conf.contents:  # was the --contents CLI option used?
            if not self.conf.scan:  # and what about the --scan option?
                self.module_set_dirname = self.get_default_module_set_dirname()
                if not self.module_set_dirname:
                    return ReturnValues.SCENARIO
            else:
                self.module_set_dirname = self.conf.scan
            self.module_set_path = os.path.join(self.conf.source_dir,
                                                self.module_set_dirname)
            if not os.path.isdir(self.module_set_path):
                log_message("Module set '%s' is not installed.\nFor a list"
                            " of installed module sets, use -l option."
                            % self.module_set_dirname, level=logging.ERROR)
                return ReturnValues.SCENARIO
            self.all_xccdf_xml_path = os.path.join(
                self.module_set_path, settings.all_xccdf_xml_filename)
        else:
            self.all_xccdf_xml_path = os.path.abspath(self.conf.contents)
            self.module_set_path = os.path.dirname(self.all_xccdf_xml_path)
            self.module_set_dirname = os.path.basename(self.module_set_path)
            if not self.module_set_dirname:
                log_message("Unable to determine a module set directory name.",
                            level=logging.ERROR)
                log_message("The directory name needs to be in format"
                            " 'RHELx_y' for upgrades from RHEL x to RHEL y.",
                            level=logging.ERROR)
                return ReturnValues.SCENARIO
        if not os.path.isfile(self.all_xccdf_xml_path):
            log_message("'%s' does not exist." % self.all_xccdf_xml_path,
                        level=logging.ERROR)
            return ReturnValues.SCENARIO

    def determine_module_set_copy_location(self):
        """The module set to be used for the system assessment will needs to be
        copied into the system assessment working directory. Determine the
        location of the copied module set.
        """
        self.module_set_copy_path = os.path.join(
            settings.assessment_results_dir,
            self.rename_custom_module_set(self.module_set_dirname))
        self.all_xccdf_xml_copy_path = os.path.join(
            self.module_set_copy_path, settings.all_xccdf_xml_filename)

    def run(self):
        """run analysis"""
        version_msg = "Preupgrade Assistant version: %s" % VERSION
        if self.conf.version:
            print (version_msg)
            return 0

        logger_debug.debug(version_msg)
        if self.conf.list_contents_set:
            for dir_name, dummy_content in iter(get_installed_module_sets(
                    self.conf.source_dir).items()):
                log_message("%s" % dir_name)
            return 0

        if self.conf.riskcheck:
            result_xml_path = os.path.join(settings.assessment_results_dir,
                                           settings.xml_result_name)
            if not os.path.exists(result_xml_path):
                log_message("System assessment needs to be performed first.")
                return ReturnValues.PREUPG_BEFORE_RISKCHECK
            return XccdfHelper.check_inplace_risk(result_xml_path,
                                                  self.conf.verbose)

        if self.conf.upload and self.conf.results:
            if not self.upload_results():
                return ReturnValues.SEND_REPORT_TO_UI
            return 0

        if self.conf.cleanup:
            if not self.executed_under_root():
                return ReturnValues.ROOT
            self.clean_preupgrade_environment()
            return 0

        if self.conf.text:
            # Test whether w3m, lynx and elinks packages are installed
            found = False
            for pkg in SystemIdentification.get_convertors():
                if xml_manager.get_package_version(pkg):
                    self.text_convertor = pkg
                    found = True
                    break
            if not found:
                log_message(settings.converter_message.format(
                    ' '.join(SystemIdentification.get_convertors())))
                return ReturnValues.MISSING_TEXT_CONVERTOR

        return_code = self.determine_module_set_location()
        if return_code:
            return return_code
        self.determine_module_set_copy_location()

        if self.conf.list_rules:
            rules = [x for x in
                     XccdfHelper.get_list_rules(self.all_xccdf_xml_path)]
            log_message('\n'.join(rules))
            return 0

        if self.conf.mode and self.conf.select_rules:
            log_message(settings.options_not_allowed)
            return ReturnValues.MODE_SELECT_RULES

        # If force option is not mentioned and user selects NO then exit
        if not self.conf.force:
            text = ""
            if self.conf.dst_arch:
                correct_option = [x for x in settings.migration_options
                                  if self.conf.dst_arch == x]
                if not correct_option:
                    sys.stderr.write(
                        "Error: Specify correct value for --dst-arch"
                        " option.\nValid are: %s.\n"
                        % ", ".join(settings.migration_options)
                    )
                    return ReturnValues.INVALID_CLI_OPTION
            if SystemIdentification.get_arch() == "i386" or \
                    SystemIdentification.get_arch() == "i686":
                if not self.conf.dst_arch:
                    text = '\n' + settings.migration_text
            logger_debug.debug("Architecture '%s'. Text '%s'.",
                               SystemIdentification.get_arch(), text)
            if not show_message(settings.warning_text + text):
                # User does not want to continue
                return ReturnValues.USER_ABORT

        self.openscap_helper = OpenSCAPHelper(self.conf.assessment_results_dir,
                                              self.conf.result_prefix,
                                              self.conf.xml_result_name,
                                              self.conf.html_result_name,
                                              self.all_xccdf_xml_path)

        if not self.executed_under_root():
            return ReturnValues.ROOT
        if not os.path.exists(settings.openscap_binary):
            log_message("Oscap with SCE enabled is not installed")
            return ReturnValues.MISSING_OPENSCAP
        if not os.access(settings.openscap_binary, os.X_OK):
            log_message("Oscap with SCE %s is not executable"
                        % settings.openscap_binary)
            return ReturnValues.MISSING_OPENSCAP

        self.execution_dir = os.getcwd()
        os.chdir("/tmp")
        retval = self.scan_system()
        if retval != 0:
            return retval
        retval = self.summary_report(self.tar_ball_name)
        self.common.copy_common_files()
        KickstartGenerator.kickstart_scripts()
        FileHelper.remove_home_issues()
        if self.conf.upload:
            if not self.upload_results():
                retval = ReturnValues.SEND_REPORT_TO_UI
        os.chdir(self.execution_dir)
        return retval

    def executed_under_root(self):
        if os.geteuid() != 0:
            print("Need to be root", end="\n")
            if not self.conf.debug:
                return False
        return True



# -*- coding: utf-8 -*-

"""
admin.py

Auto-register admin classes with fields and links to linked model classes

based on http://djangosnippets.org/snippets/997/

Sample:

get_group_state = lambda self, instance: \
    GROUP_STATES.get_value(instance.group_state)
defect_states.short_description = 'Group state'

register_admin_module('project.app.models', new_fields={
    'Group': ('group_state_field', get_group_state),
})


"""

from django.core.urlresolvers import reverse
from django.utils.safestring import mark_safe
from django.contrib import admin
from django.db import models as dmodels
from django.db.models import Field, ForeignKey, OneToOneField
from preupg.ui.utils import public
from types import ModuleType


def add_link_field(admin_class, field):
    field_name = field.name + '_link'

    def link(self, instance):
        app_name = field.related.parent_model._meta.app_label
        reverse_path = "admin:%s_%s_change" % (
            app_name,
            field.related.parent_model._meta.module_name
        )
        related_instance = getattr(instance, field.name)
        # it might point to None (foreignKey(null=True))
        if related_instance:
            url = reverse(reverse_path, args=(related_instance.id,))
            return mark_safe("<a href='%s'>%s</a>" % (
                url,
                unicode(related_instance))
            )
        else:
            return unicode(related_instance)
    link.allow_tags = True
    link.short_description = field.name + ' link'
    setattr(admin_class, field_name, link)
    admin_class.readonly_fields = list(
        getattr(admin_class, 'readonly_fields', [])) + [field_name]
    return admin_class, field_name


@public
def register_admin_module(module, exclude=None, new_fields=None,
                          search_fields=None):
    """
    @param module: module containing django.db.models classes
    @type module: str or __module__
        If you are providing str, use absolute path

    @param exclude: list of classes to exclude from auto-register
    @type exclude: iterable of str or None
    @param new_fields: dictionary of additional fields:
        {'model name': ('field_name', callable)}
    @type new_fields: dict or None
    @param search_fields: list of tuples to search in:
        ('model name', ['list', 'of', 'fields'])
    @type search_fields: dict or None
    """
    exclude = exclude or []
    new_fields = new_fields or {}
    search_fields = search_fields or {}
    if isinstance(module, ModuleType):
        models = module
    elif isinstance(module, basestring):
        # import module dynamically -- import leaf, not root
        models = __import__(module, fromlist=[module.split('.')[-1]])
    else:
        raise TypeError("invalid type of argument 'module', expected 'str' or \
'ModuleType', got %s." % type(module))

    #get models from current app
    mods = []
    for x in models.__dict__.values():
        if issubclass(type(x), dmodels.base.ModelBase) and \
                getattr(x, '__name__', '') not in exclude:
            mods.append(x)

    admins = []
    #for each model prepare an admin class (Admin<model_name>, model)
    for c in mods:
        admins.append(("%sAdmin" % c.__name__, c))

    #create the admin class and register it
    for (ac, c) in admins:
        admin_class = type(ac, (admin.ModelAdmin,), dict())
        admin_class.list_display = []
        for field in c._meta.fields:
            field_name = field.name
            # create link for relations
            if issubclass(type(field), (ForeignKey, OneToOneField)):
                admin_class, field_name = add_link_field(admin_class, field)
            if issubclass(type(field), (ForeignKey, OneToOneField, Field)):
                admin_class.list_display.append(field_name)

        # add user defined custom fields
        for new_field in new_fields.iterkeys():
            if c.__name__ == new_field:
                setattr(admin_class, new_fields[new_field][0],
                        new_fields[new_field][1])
                admin_class.list_display.append(new_fields[new_field][0])

        # add fields to search in
        for model_name, fields_to_search in search_fields:
            if c.__name__ == model_name:
                setattr(admin_class, 'search_fields', fields_to_search)

        try:  # pass gracefully on duplicate registration errors
            admin.site.register(c, admin_class)
        except Exception:
            pass



# -*- coding: utf-8 -*-
"""
Functions for processing reports -- extracting data from XML documents
"""

from __future__ import print_function
from datetime import datetime
import logging

import re

from xml.etree import ElementTree

logger = logging.getLogger('preup_ui')


def xml_to_html(xml_str):
    """
    XML can't be easily rendered as HTML, so we need to get rid of
    namespaces, prefixes...
    """
    # get rid of all the awesome xml stuff
    # <?xml version='1.0' encoding='UTF-8'?>\n
    xml_str = re.sub('<\?xml.+\?>', '', xml_str)
    # <html:{br,p,ul,li} xmlns:html="http://www.w3.org/1999/xhtml" href=""{ /,}>
    # ditch xmlns first
    xml_str = re.sub(' xmlns:\w+?="[^"]+?"', r'', xml_str)
    # now remove tag's prefix: *:<tag>
    xml_str = re.sub('<(/?)\w*:(\w+)', r'<\1\2', xml_str)

    # fix relative link to be served by webserver
    # <a href="file:./kickstart/untrackedsystem" ...
    xml_str = re.sub(
        '<a href="(./|file:)([^"]+)"\s*>',
        r'<a href="__INSERT_URL__?path=\2" target="_blank">',
        xml_str)
    xml_str = xml_str.strip('\n')
    return xml_str


def stringify_children(node):
    if node is None or node.text is None:
        return ''
    # strip newlines added by xmlparser
    parts = [node.text.strip('\n')]

    # was node.getchildren()
    for c in node:
        # 'method' argument is not present on python-2.6:
        #   method="html"
        child_str = ElementTree.tostring(c, encoding="UTF-8")

        child_str = xml_to_html(child_str)

        parts.append(child_str)
    # filter removes possible Nones in texts and tails
    response = ''.join(filter(None, parts)).strip()
    return response


def get_nodes(tree, tag, ns='', prefix=''):
    return tree.findall(prefix + ns + tag)


def get_node(tree, tag, ns='', prefix=''):
    return tree.find(prefix + ns + tag)


def set_if_true(d, key, value):
    """ shortcut for: `if value: d[key] = value` """
    if value is not None and len(value) > 0:
        d[key] = value


class XMLReportParser(object):

    def __init__(self, report_path):
        """
        run['groups'] = [
            {
                'xccdf_id': '', 'name': '', 'rules':
                    [{'id_ref': '', 'title': '', ...}, {...}, ]
            }
        ]
        """
        self.path = report_path
        # data structure with every information
        self.run = {'groups': [], }
        # 'symlink' to rules
        self.rules = []
        # everyone loves XML
        self.element_prefix = "{http://checklists.nist.gov/xccdf/1.2}"

    def get_child(self, tree, tag):
        return get_node(tree, tag, self.element_prefix, prefix='./')

    def filter_children(self, tree, tag):
        return get_nodes(tree, tag, self.element_prefix, prefix='./')

    def has_children(self, tree, tag):
        return len(get_nodes(tree, tag, self.element_prefix, prefix='./')) > 0

    def filter_grandchildren(self, tree, parent_tag, tag):
        return tree.findall('./%s%s/%s%s' % (self.element_prefix, parent_tag, self.element_prefix, tag))

    def get_nodes_atrib(self, tree, tag, attrib):
        try:
            return self.get_child(tree, tag).attrib[attrib]
        except (KeyError, AttributeError):
            return ''

    def get_nodes_text(self, tree, tag):
        try:
            text = self.get_child(tree, tag).text
        except AttributeError:
            pass
        else:
            if text:
                return text.strip()
        return ''

    def get_test(self, key):
        """ this may throw IndexError if test is not in self.run """
        #try:
        return filter(lambda x: x['id_ref'] == key, self.rules)[0]
        #except IndexError:
        #    return ''

    def parse_rules(self, root):
        if root is None:
            return []
        rules = []
        for rule in self.filter_grandchildren(root, 'Group', 'Rule'):
            test = {}
            test['id_ref'] = rule.attrib['id']
            set_if_true(test, 'title', self.get_nodes_text(rule, 'title'))
            set_if_true(test, 'description',
                        stringify_children(self.get_child(rule, 'description')))
            set_if_true(test, 'fix', self.get_nodes_text(rule, 'fix'))
            set_if_true(test, 'fixtext', stringify_children(self.get_child(rule, 'fixtext')))
            set_if_true(test, 'fix_type',
                        self.get_nodes_atrib(rule, 'fix', 'system'))
            rules.append(test)
            self.rules.append(test)
        return rules

    def parse_groups(self, root, parent=None):
        """
        parse groups (sets of tests)

        This method is probably most tricky. It iterates over all groups recursively.
        If there is grandchild <Rule> it will extract rules.
        If there is grandchild <Group> it will recursively process it.
        """
        def has_grandchild(root_elem, parent_tag, tag):
            g = self.filter_grandchildren(root_elem, parent_tag, tag)
            return len(g) > 0

        groups = self.filter_children(root, 'Group')

        # top level groups
        for group in groups:
            if parent and not self.has_children(group, 'Group'):
                # ^this is tricky, we are facing situation where this group is not root group;
                # the problem is, that every rule has its own group so we need
                # to filter only groups which have new unprocessed tests
                continue
            group_dict = {}
            group_dict['xccdf_id'] = group.attrib['id']
            group_dict['title'] = self.get_nodes_text(group, 'title')
            set_if_true(group_dict, 'parent', parent)
            group_dict.setdefault('rules', [])

            if has_grandchild(group, 'Group', 'Rule'):
                group_dict['rules'] = self.parse_rules(group)

            self.run['groups'].append(group_dict)
            # THIS HAS TO BE AFTER ADDITION OF CURRENT GROUP (chicken-egg problem)
            if has_grandchild(group, 'Group', 'Group'):
                self.parse_groups(group, group_dict['xccdf_id'])


    def parse_test_result_logs(self, text):
        """
        parse test's logs; result is list of dicts:
        [
            {'level': '', 'date': '', 'message': ''}
        ]
        """
        if not text:
            return None, None
        text = text.strip()
        lines = text.split('\n')

        log_regex = "preupg\.log\.(?P<level>(ERROR|WARNING|INFO|DEBUG)): (?P<date_str>\S+) (?P<time>\S+) (?P<message>.+)"
        risk_regex = "preupg\.risk\.(?P<level>\w+): (?P<message>.+)"
        date_format = '%Y-%m-%d %H:%M'
        logs = []
        risks = []
        for line in lines:
            match = re.match(log_regex, line)
            if match:
                match_dict = match.groupdict()
                try:
                    dt = match_dict['date_str'] + ' ' + match_dict['time']
                except KeyError:
                    pass
                else:
                    try:
                        match_dict['date'] = datetime.strptime(dt, date_format)
                    except ValueError:
                        match_dict['date'] = None
                logs.append(match_dict)
            else:
                match = re.match(risk_regex, line)
                if match:
                    match_dict = match.groupdict()
                    risks.append(match_dict)
        return logs, risks

    def get_test_result_logs(self, elem):
        """ retrieve test's logs from xml """
        # python-2.6: find doesnt know 'name[.*]'
        # selector = 'check-import[@import-name="stdout"]'
        # text = self.get_nodes_text(elem, selector)
        if elem is None:
            return None, None
        nodes = self.filter_children(elem, 'check-import')
        found = False
        for n in nodes:
            if n.attrib.get('import-name') == 'stdout':
                found = True
                break
        if not found:
            return None, None
        text = n.text
        parsed_logs, parsed_risks = self.parse_test_result_logs(text)
        return parsed_logs, parsed_risks

    def parse_rule_results(self, root):
        """ parse info about each test result """
        # element.iter is not on python-2.6
        #for result in root.iter(self.element_prefix + 'rule-result'):
        for result in root.findall('.//' + self.element_prefix + 'rule-result'):
            result_state = self.get_nodes_text(result, 'result')
            idref = result.attrib['idref']
            if result_state in ['error', 'notchecked']:
                logger.error("Test %s crashed.", idref)
            if result_state not in ['notselected']:
                try:
                    test = self.get_test(idref)
                except IndexError:
                    logger.error("Test %s not found", idref)
                else:
                    set_if_true(test, 'result', result_state)
                    set_if_true(test, 'time', result.attrib['time'])

                    # test logs are in element check/check-import[@import-name=stdout]
                    check_elem = self.get_child(result, 'check')
                    if check_elem is not None:
                        parsed_logs, parsed_risks = self.get_test_result_logs(check_elem)
                        set_if_true(test, 'logs', parsed_logs)
                        set_if_true(test, 'risks', parsed_risks)

    def process_run_info(self, root):
        """ get information about run and info about host """
        tr = self.get_child(root, 'TestResult')
        self.run['host'] = self.get_nodes_text(tr, 'target')
        self.run['identity'] = self.get_nodes_text(tr, 'identity')
        self.run['addresses'] = []
        if tr:
            for address in get_nodes(tr, 'target-address', self.element_prefix):
                self.run['addresses'].append(address.text)
            self.run['started'] = tr.attrib['start-time']
            self.run['finished'] = tr.attrib['end-time']
            logger.debug("Started: %s, Finished: %s", self.run['started'], self.run['finished'])
        logger.debug("Host: %s, Identity: %s", self.run['host'], self.run['identity'])

    def parse_report(self):
        """ parse XML report """
        root = ElementTree.parse(self.path).getroot()
        # rules & groups first
        self.parse_groups(root)
        self.parse_rule_results(root)
        self.process_run_info(root)
        return self.run


def parse_xml_report(xml_filepath):
    r = XMLReportParser(xml_filepath)
    return r.parse_report()


def update_html_report(html_filepath):
    """Links to files and folders need to be updated to work in the Web UI."""
    with open(html_filepath, 'r') as infile:
        updated_html = re.sub(r'<a href="(./|file:)([^"]+)"\s*>',
                              r'<a href="../file/?path=\2" target="_blank">',
                              infile.read())
    with open(html_filepath, 'w') as outfile:
        outfile.write(updated_html)


def main():
    from pprint import pprint
    import sys
    try:
        r = parse_xml_report(sys.argv[1])
    except KeyError:
        print ('Usage: prog <content.xml>')
        sys.exit(1)
    pprint(r)


if __name__ == '__main__':
    main()



temp_bash_script = """#!/bin/bash

. /usr/share/preupgrade/common.sh

#END GENERATED SECTION

### For more information see 'man preupg-content-creator' or 'man preupgrade-assistant-api'.

"""

temp_python_script = """#!/usr/bin/python
# -*- Mode: Python; python-indent: 8; indent-tabs-mode: t -*-

import sys
import os

from preupg.script_api import *

#END GENERATED SECTION

### For more information see 'man preupg-content-creator' or 'man preupgrade-assistant-api'.
"""
default_group = 'system'
default_module = 'packages'
group_name = 'Specify the group name which the module belongs to [%s]:' % default_group
content_name = 'Specify the module name which will be created [%s]:' % default_module
content_title = "Specify the module title:"
content_desc_text = "Specify the module description:"
upgrade_path = "Specify the name of the module set directory in which the module will be created:"
summary_title = 'preupg-content-creator generated these files to be updated:'
summary_directory = '- the module was created in the %s directory.'
summary_ini = '- the INI file which defines the module is %s.'
summary_check = '- the check script which provides an assessment is %s. Update it before you use it.'
summary_solution = '- the solution text which informs about incompatibilities %s. Update it before you use it.'
check_path = "The %s file already exists. Do you want to replace the file?"
type_check_script = "Would you like to create a BASH or Python check script? [sh/py] Bash is default."
prop_src_version = "Specify major source OS version e.g. \"6\":"
prop_dst_version = "Specify major destination OS version e.g. \"7\":"
commands_to_use_new_module = "\nTo use the newly created module with the" \
                             " Preupgrade Assistant run these commands:\n" \
                             "- preupg-xccdf-compose %s\n" \
                             "- preupg -c %s"



from __future__ import print_function, unicode_literals
import os
import sys
import re
import datetime
import shutil

from distutils import dir_util

from preupg.utils import FileHelper, ModuleSetUtils
from preupg.xmlgen.oscap_group_xml import OscapGroupXml
from preupg import settings
from preupg import xccdf
from preupg.logger import logger_debug
from preupg.settings import ReturnValues
from preupg.logger import log_message, logging

try:
    from xml.etree import ElementTree
except ImportError:
    from elementtree import ElementTree
try:
    from xml.etree.ElementTree import ParseError
except ImportError:
    from xml.parsers.expat import ExpatError as ParseError

XCCDF_FRAGMENT = "{http://fedorahosted.org/sce-community-content/wiki/" \
                 "XCCDF-fragment}"
SCE = "http://open-scap.org/page/SCE"


class XCCDFCompose(object):
    """
    Prepare result directory and take care of creating all-xccdf.xml file
    """

    def __init__(self, src_path, dst_path=None):
        """
        Create the XCCDFCompose object with specified src and dst path.

        The src_path specify source modules from which the XCCDF compose should
        be created on the dst_path. In case the dst_path is not specified, the
        result directory will be created, in the same place the source directory
        exists, with the "-results" suffix using the original dirname.

        src_path and dst_path has to be different, otherwise ValueError
        exception is raised.
        """
        self.src_path = src_path
        if not dst_path:
            self.dst_path = self.src_path + settings.results_postfix
            if self.src_path.endswith("/"):
                self.dst_path = self.src_path[:-1] + settings.results_postfix
        else:
            self.dst_path = dst_path
        if self.dst_path == self.src_path:
            raise ValueError("src_path and dst_path has to be different.")

        # Delete previous contents if they exist.
        if os.path.exists(self.dst_path):
            shutil.rmtree(self.dst_path)

    def generate_xml(self, generate_from_ini=True):
        """
        Copy files to result directory and if specified generate all-xccdf.xml
        file

        @param {bool} generate_from_ini - True if xccdf-compose tool is used,
            decide if all-xccdf.xml file will(True) be created or not(False)
        @throws {IOError} - when creation of all-xccdf.xml file fails
        @return {int} - 0 success
                       !0 error
        """
        try:
            ModuleSetUtils.get_module_set_os_versions(self.src_path)
        except EnvironmentError as err:
            sys.stderr.write("{0}\n".format(str(err)))
            return ReturnValues.SCENARIO

        # e.g. /root/preupgrade/RHEL6_7 -> /root/preupgrade/RHEL6_7-results
        dir_util.copy_tree(self.src_path, self.dst_path)
        # create content for all-xccdf.xml file as ElementTree object
        target_tree = ComposeXML.run_compose(
            self.dst_path, generate_from_ini=generate_from_ini)
        # path where all-xccdf.xml is going to be generated
        report_filename = os.path.join(self.dst_path,
                                       settings.all_xccdf_xml_filename)
        if generate_from_ini:
            try:
                FileHelper.write_to_file(
                    report_filename, "wb",
                    ElementTree.tostring(target_tree, "utf-8"), False
                )
                logger_debug.debug('Generated: %s' % report_filename)
            except IOError:
                raise IOError("Error: Problem with writing file %s"
                              % report_filename)
        return 0

    def get_compose_dir_name(self):
        return self.dst_path


class ComposeXML(object):

    @staticmethod
    def collect_group_xmls(module_set_dir, source_dir, generate_from_ini=True):
        """
        Find group.xml file recursively through all module directories
        and modules. Collect data from each of them into dictionary.

        @param {str} module_set_dir - directory where all modules are stored
        @param {str} source_dir - directory path for processing
        @param {bool} generate_from_ini - True if xccdf-compose tool is used

        @return {dict} - structure is file based, keys are top level module
        directories, values are tuples which consist of 2 elements:
            [0] - XML data for top level dir itself
            [1] - dict of all modules inside [0] directory
        @example
        from file structure:
            - RHEL6_7
                - services (modules directory)
                    - group.xml
                    - group.ini
                    - httpd (module)
                        - group.xml
                        - ...
                    - tomcat (module)
                        - group.xml
                        - ...
                - drivers (modules directory)
                - ...
        result is:
            {
                services: tuple(
                    <Element {services}>,
                    {
                        httpd: tuple(<Element {httpd}>, {}),
                        tomcat: tuple(<Element {tomcat}>, {}),
                        ...
                    }
                ),
                drivers: tuple(
                    <Element {drivers}>,
                    {...}
                ),
                ...
            }
        """
        ret = {}

        for dirname in os.listdir(source_dir):
            if dirname and dirname[0] == '.':
                continue
            new_dir = os.path.join(source_dir, dirname)
            if not os.path.isdir(new_dir):
                continue
            ini_files = [x for x in os.listdir(new_dir) if x.endswith('.ini')]
            if not ini_files and generate_from_ini:
                # Check if directory contains only subdirectories.
                # Report to user that group.ini file could be missing
                directories = [x for x in os.listdir(new_dir)
                               if not os.path.isdir(os.path.join(new_dir, x))]
                if not directories and 'postupgrade.d' not in dirname:
                    log_message(
                        "group.ini file is missing in {0}".format(new_dir),
                        level=logging.WARNING)
            if ini_files and generate_from_ini:
                oscap_group = OscapGroupXml(module_set_dir, new_dir)
                oscap_group.write_xml()
                return_list = oscap_group.collect_group_xmls()
                ComposeXML.perform_autoqa(new_dir, return_list)

            group_file_path = os.path.join(new_dir, "group.xml")
            if not os.path.isfile(group_file_path):
                continue
            try:
                ret[dirname] = (ElementTree.parse(group_file_path).getroot(),
                                ComposeXML.collect_group_xmls(
                                    module_set_dir, new_dir,
                                    generate_from_ini))
            except ParseError as e:
                log_message(
                    "Encountered a parse error in {0} file, details: {1}"
                    .format(group_file_path, e), level=logging.ERROR)
                sys.exit(1)
        return ret

    @staticmethod
    def perform_autoqa(path_prefix, group_tree):
        for f, t in iter(group_tree.items()):
            b_subgroups = True
            try:
                tree, subgroups = t
            except ValueError:
                tree = t
                b_subgroups = False

            group_xml_path = os.path.join(f, "group.xml")

            groups = tree.findall(xccdf.XMLNS + "Group")
            if len(groups) != 1:
                continue

            for element in tree.findall(".//" + xccdf.XMLNS + "Rule"):
                checks = element.findall(xccdf.XMLNS + "check")
                if len(checks) != 1:
                    print ("Rule of id ", element.get("id", ""),
                           " from ", group_xml_path,
                           " doesn't have exactly one check element!")
                    continue

                check = checks[0]

                if check.get("system") != SCE:
                    print ("Rule of id '", element.get("id", ""),
                           "' from ", group_xml_path, " has system name"
                           " different from the SCE system name ",
                           "('", SCE, "')!")

                crefs = check.findall(xccdf.XMLNS + "check-content-ref")
                if len(crefs) != 1:
                    print("Rule of id '", element.get("id", ""),
                          "' from '", group_xml_path,
                          "' doesn't have exactly one check-content-ref inside"
                          " its check element!")
                    continue

                # Check if the description contains a list of affected files
                description = element.find(xccdf.XMLNS + "description")
                if description is None:
                    print ("Rule ", element.get("id", ""),
                           " missing a description")
                    continue

            if b_subgroups:
                ComposeXML.perform_autoqa(os.path.join(path_prefix, f),
                                          subgroups)

    @staticmethod
    def repath_group_xml_tree(source_dir, new_base_dir, group_tree):
        """
        Fix href attribute inside <check-content-ref> for each
        XML Element module

        @param {str} source_dir - directory where module is located e.g.
            /root/preupgrade/RHEL6_7-results/backup/bacula
        @param {str} new_base_dir - base directory e.g.
            /root/preupgrade/RHEL6_7-results
        @param {dict} group_tree - if source_dir is module set directory it
            has all modules which are inside this directory, e.g.
            source_dir: /root/preupgrade/RHEL6_7-results/backup
            group_tree: {bacula: (<Element {bacula}>, {})}

        @return None

        @example
        from:
            <ns0:check-content-ref href="check" />
        to:
            <ns0:check-content-ref href="backup/bacula/check" />
        """
        for f, t in iter(group_tree.items()):
            tree, subgroups = t

            old_base_dir = os.path.join(source_dir, f)

            path_prefix = os.path.relpath(old_base_dir, new_base_dir)
            for element in tree.findall(".//" + xccdf.XMLNS +
                                        "check-content-ref"):
                old_href = element.get("href")
                assert(old_href is not None)
                element.set("href", os.path.join(path_prefix, old_href))

            ComposeXML.repath_group_xml_tree(old_base_dir, new_base_dir,
                                             subgroups)

    @staticmethod
    def merge_trees(target_tree, target_element, group_tree):
        def get_sorting_key_for_tree(group_tree, tree_key):
            prefix = 100
            tree, dummy_subgroups = group_tree[tree_key]
            try:
                prefix = int(tree.findall(XCCDF_FRAGMENT + "sort-prefix")
                             [-1].text)
            except:
                pass

            return prefix, tree_key

        def sort_key(t_key):
            return get_sorting_key_for_tree(group_tree, t_key)

        for f in sorted(iter(group_tree.keys()), key=sort_key):
            t = group_tree[f]
            tree, subgroups = t

            groups = tree.findall(xccdf.XMLNS + "Group")
            if len(groups) != 1:
                print("There are %i groups in '%s/group.xml' file. Exactly 1"
                      " group is expected! Skipping..." % (len(groups), f))
                continue
            target_element.append(groups[0])
            for child in tree.findall(xccdf.XMLNS + "Profile"):
                assert(child.get("id") is not None)
                merged = False

                # look through profiles in the template XCCDF
                for profile in target_tree.findall(xccdf.XMLNS + "Profile"):
                    if profile.get("id") == child.get("id"):
                        for profile_child in child.findall("*"):
                            profile.append(profile_child)

                        merged = True
                        break

                if not merged:
                    print("Found profile of id '%s' that doesn't match any"
                          " profiles in template, skipped!"
                          % (child.get("id")), sys.stderr)

            ComposeXML.merge_trees(target_tree, groups[0], subgroups)

    @staticmethod
    def resolve_selects(target_tree):
        default_selected_rules = set([])
        all_rules = set([])

        for profile in target_tree.findall(xccdf.XMLNS + "Profile"):
            selected_rules = set(default_selected_rules)

            # to avoid invalidating iterators
            to_remove = []
            for select in profile.findall("*"):

                if select.tag == xccdf.XMLNS + "select":
                    if select.get("selected", "false") == "true":
                        selected_rules.add(select.get("idref", ""))
                    else:
                        selected_rules.remove(select.get("idref", ""))

                    to_remove.append(select)

                elif select.tag == XCCDF_FRAGMENT + "meta-select":
                    needle = select.get("idref")
                    for rule_id in all_rules:
                        if re.match(needle, rule_id):
                            if select.get("selected", "false") == "true":
                                selected_rules.add(rule_id)
                            elif rule_id in selected_rules:
                                selected_rules.remove(rule_id)

                    to_remove.append(select)

            for rm in to_remove:
                profile.remove(rm)

            for rule in selected_rules:
                # if it's selected by default, we don't care
                if rule not in default_selected_rules:
                    elem = ElementTree.Element(xccdf.XMLNS + "select")
                    elem.set("idref", rule)
                    elem.set("selected", "true")
                    profile.append(elem)

            for rule in default_selected_rules:
                if rule not in selected_rules:
                    elem = ElementTree.Element(xccdf.XMLNS + "select")
                    elem.set("idref", rule)
                    elem.set("selected", "false")
                    profile.append(elem)

    @staticmethod
    def refresh_status(target_tree):
        for status in target_tree.findall(xccdf.XMLNS + "status"):
            if status.get("date", "") == "${CURRENT_DATE}":
                status.set("date", datetime.date.today().strftime("%Y-%m-%d"))

    # taken from http://effbot.org/zone/element-lib.htm#prettyprint
    @staticmethod
    def indent(elem, level=0):
        i = "\n" + level * "  "
        if len(elem):
            if not elem.text or not elem.text.strip():
                elem.text = i + "  "
            if not elem.tail or not elem.tail.strip():
                elem.tail = i
            for elem in elem:
                ComposeXML.indent(elem, level + 1)
            if not elem.tail or not elem.tail.strip():
                elem.tail = i
        else:
            if level and (not elem.tail or not elem.tail.strip()):
                elem.tail = i

    @staticmethod
    def get_template_file():
        return os.path.join(settings.data_dir, "templates",
                            settings.xccdf_template)

    @staticmethod
    def get_xml_tree():
        template_file = ComposeXML.get_template_file()
        try:
            target_tree = ElementTree.parse(template_file).getroot()
        except IOError:
            raise IOError('Error: Problem with reading %s file'
                          % template_file)
        return target_tree

    @staticmethod
    def run_compose(dir_name, generate_from_ini=True):
        target_tree = ComposeXML.get_xml_tree()
        settings.UPGRADE_PATH = dir_name
        if os.path.exists(os.path.join(dir_name, settings.file_list_rules)):
            os.unlink(os.path.join(dir_name, settings.file_list_rules))
        group_xmls = ComposeXML.collect_group_xmls(dir_name, dir_name,
                                                   generate_from_ini)
        logger_debug.debug("Group xmls '%s'", group_xmls)
        if generate_from_ini:
            ComposeXML.perform_autoqa(dir_name, group_xmls)
        new_base_dir = dir_name
        ComposeXML.repath_group_xml_tree(dir_name, new_base_dir, group_xmls)
        ComposeXML.merge_trees(target_tree, target_tree, group_xmls)
        ComposeXML.resolve_selects(target_tree)
        ComposeXML.refresh_status(target_tree)
        ComposeXML.indent(target_tree)

        return target_tree



# -*- coding: utf-8 -*-
"""
utils functions for rendering and displaying data
"""

import re

from django.http.response import Http404, HttpResponseForbidden
from django.template import RequestContext, loader


def return_error(request, message):
    context = RequestContext(request,
        {'message': message}
    )
    template = loader.get_template('error.html')
    return HttpResponseForbidden(template.render(context))


def state_filter_form_field_name(state, result_id):
    return state + str(result_id)

def is_state_filter(field_name):
    """
    is provided string a field in form for filtering by state?
    return state and result ID
    """
    m = re.match('(\w+?)(\d+)', field_name)
    if m:
        return m.groups()
    else:
        return None

def get_states_to_filter(get):
    if not get:
        return []
    states = []
    for key in get.keys():
        if get[key] == u'on':
            m = is_state_filter(key)
            if m is not None:
                states.append(m[0])
    return states



from django.contrib.auth.backends import ModelBackend

from ..config.models import AppSettings

class AutologinBackend(ModelBackend):
    """
    Auto authenticate user.
    """

    def authenticate(self, **credentials):
        """
        returns user to be auto-logged in or None
        """
        user_id = AppSettings.get_autologin_user_id()
        return user_id and self.get_user(user_id)




import unittest

try:
    import base
except ImportError:
    import tests.base as base
import inspect
import os
import sys

from preupg import preupg_diff


class TestPreupgDiff(base.TestCase):

    current_dir = os.path.dirname(inspect.getfile(inspect.currentframe()))

    class argv_mocked(list):
        pass

    class dummy_mock(base.MockFunction):
        def __call__(self, *args, **kwargs):
            pass

    class print_difference_status_mock(base.MockFunction):
        def __call__(self, num_new_xml_rules, num_diff_xml_rules):
            self.num_new_rules = num_new_xml_rules
            self.num_diff_rules = num_diff_xml_rules

    @base.mock(sys, "argv", argv_mocked)
    @base.mock(preupg_diff, "save_diff_to_xml_and_html_file", dummy_mock())
    @base.mock(preupg_diff, "print_difference_status",
               print_difference_status_mock())
    def test_xml_on_input(self):
        sys.argv = [
            "preupg-diff",
            os.path.join(self.current_dir,
                         "generated_results",
                         "inplace_combined_risk_test.xml"),
            os.path.join(self.current_dir,
                         "generated_results",
                         "inplace_risk_test.xml")]

        preupg_diff.run()

        self.assertEqual(preupg_diff.print_difference_status.num_new_rules, 2)
        self.assertEqual(preupg_diff.print_difference_status.num_diff_rules, 1)

    class stringify_children_mock(base.MockFunction):
        def __call__(self, tag_obj):
            self.children_as_str = preupg_diff.stringify_children_orig(tag_obj)
            return self.children_as_str

    @base.mock(preupg_diff, "stringify_children", stringify_children_mock())
    def test_stringify_solution_text(self):
        preupg_diff.ResultXML(os.path.join(self.current_dir,
                                           "generated_results",
                                           "inplace_risk_test.xml"))
        self.assertEqual(preupg_diff.stringify_children.children_as_str,
                         'str1<br xmlns:xhtml="http://www.w3.org/1999/xhtml/"'
                         ' xmlns:ns0="http://checklists.nist.gov/xccdf/1.2"/>'
                         'str2')


def suite():
    loader = unittest.TestLoader()
    suite = unittest.TestSuite()
    suite.addTest(loader.loadTestsFromTestCase(TestPreupgDiff))
    return suite


if __name__ == "__main__":
    unittest.TextTestRunner(verbosity=3).run(suite())



#!/usr/bin/python2
# -*- coding: utf-8 -*-

import os
import re
import distutils.command.sdist
from distutils.command.install import INSTALL_SCHEMES
from setuptools import setup, find_packages
from preupg.version import VERSION
from preupg.settings import DOC_DIR

project_name = "preupgrade-assistant"
project_url = "https://github.com/upgrades-migrations/preupgrade-assistant/"
project_author = "Red Hat, Inc."
project_author_email = "phracek@redhat.com"
project_description = "Preupgrade Assistant"
package_name = "%s" % project_name
package_module_name = project_name
package_version = VERSION

script_files = ['bin/preupg', 'tools/preupg-content-creator',
                'tools/preupg-kickstart-generator', 'tools/preupg-ui-manage',
                'tools/preupg-xccdf-compose', 'tools/preupg-diff']

data_files = {
    'preupg/ui/report/fixtures/':
        ['preupg/ui/report/fixtures/initial_data.json'],
    '/usr/share/preupgrade/':
        ['common.sh'],
    DOC_DIR:
        ['LICENSE', 'doc/README', 'doc/README.kickstart', 'doc/README.ui']
}

# Include relative path to dirs with non-python files - these will be added
# to the python module directory with the same relative path
paths = ['preupg/ui/templates/', 'preupg/ui/static/']
for path in paths:
    for root, dirs, files in os.walk(path):
        data_files[root] = [os.path.join(root, f) for f in files]

# Specify absolute paths into which content from the relative path-defined
# dirs will be copied to
paths = {'etc/': '/etc',
         'data/': '/usr/share/preupgrade/data',
         'doc/module_writing_tutorial/':
             os.path.join(DOC_DIR, 'module_writing_tutorial')}
for relative_source_dir, absolute_dest_dir in iter(paths.items()):
    for root, dirs, files in os.walk(relative_source_dir):
        path_to_join = re.split(relative_source_dir + "/?", root)[1]
        absolute_dir = os.path.join(absolute_dest_dir, path_to_join)
        relative_file_paths = [os.path.join(root, f) for f in files]
        if relative_file_paths:
            data_files[absolute_dir] = relative_file_paths

# override default tarball format with bzip2
distutils.command.sdist.sdist.default_format = {'posix': 'bztar'}

packages = find_packages(exclude=['tests'])

root_dir = os.path.dirname(__file__)
if root_dir != "":
    os.chdir(root_dir)

for scheme in INSTALL_SCHEMES.values():
    scheme["data"] = scheme["purelib"]

setup(
    name=package_name,
    version=package_version,
    url=project_url,
    author=project_author,
    author_email=project_author_email,
    description=project_description,
    packages=packages,
    data_files=data_files.items(),
    scripts=script_files,
    test_suite='tests.suite'
)



import optparse


class CLICreator(object):

    """Class for processing data from commandline"""

    def __init__(self, args=None):
        """parse arguments"""
        self.parser = optparse.OptionParser()

        self.add_args()
        if args:
            self.opts, self.args = self.parser.parse_args(args=args)
        else:
            self.opts, self.args = self.parser.parse_args()

    def add_args(self):
        self.parser.add_option(
            "-m",
            "--main-directory",
            dest="maindir",
            help="Specify main directory for content",
        )
        self.parser.add_option(
            "-v",
            "--validate",
            action='store_true',
            help="Validate content(s)"
        )
        self.parser.add_option(
            "-d",
            "--debug",
            action='store_true',
            help="Turn on debugging mode"
        )

if __name__ == '__main__':
    x = CLICreator()
    print (x.args.id)



# -*- coding: utf-8 -*-

"""
Class creates a set of packages for migration scenario
"""

from preupg.utils import FileHelper
from preupg.kickstart.application import BaseKickstart


class TimezoneHandling(BaseKickstart):
    """class for replacing/updating package names"""

    def __init__(self, handler):
        """
        """
        self.handler = handler
        self.timezone = None

    @staticmethod
    def get_kickstart_timezone():
        """
        Contain of the file /etc/sysconfig/clock is ZONE='TIMEZONE'
        :return: list with timezone
        """
        timezone_file = '/etc/sysconfig/clock'
        try:
            lines = FileHelper.get_file_content(timezone_file, 'rb', method=True)
        except IOError:
            return None
        lines = [x for x in lines if x.startswith('ZONE=')]
        if not lines:
            return None
        return lines

    def update_timezone(self):
        if self.timezone:
            for line in self.timezone:
                fields = line.strip().split('=')
                try:
                    self.handler.timezone.timezone = fields[1]
                    self.handler.timezone.isUtc = True
                    break
                except KeyError:
                    pass

    def run_module(self, *args, **kwargs):
        self.timezone = self.get_kickstart_timezone()
        self.update_timezone()



# -*- coding: utf-8 -*-
import uuid
from django.core.urlresolvers import reverse
import os

from preupg.ui.report.models import Run, Host
from preupg.ui.report.service import import_report

from django.conf import settings


__all__ = (
    'upload_results',
    'submit_new',
    "ping",
)

def ping(request):
    """ server verification """
    return {'status': "OK"}

def upload_results(request, opts):
    """
    upload_results(opts)

    opts is dictionary, it has to contain these entries:
     * filename: tarball's filename
     * data: content of file
     * runhost_id: ID of run for specific host
    """
    p = os.path.join(settings.MEDIA_ROOT, opts['filename'])
    with open(p, 'wb+') as destination:
        destination.write(opts['data'].data)
    import_report(p, opts['hostrun_id'])
    return 'OK'

def submit_new(request, opts):
    """
    submit_new(opts)

    submit a new result of a run on (probably remote) host

    opts is dictionary, it has to contain these entries:
     * host: string with hostname of a host where scan was done (optional)
     * data: content of file
    """
    # as soon as the tarball will be unpacked, this die will be erased
    tmp_dir = os.path.join(settings.MEDIA_ROOT, uuid.uuid4().hex)
    try:
        os.makedirs(tmp_dir, mode=0o0744)
    except OSError as e:
        # TODO: log
        return {
            'status': 'ERROR',
            'message': 'Failed to create temporary directory: %s' % e,
        }
    report_path = os.path.join(tmp_dir, 'result.tar.gz')
    with open(report_path, 'wb+') as destination:
        destination.write(opts['data'].data)

    host, created = Host.objects.get_or_create(hostname=opts['host'])
    run_object = Run.objects.create_for_host(host)
    hostrun = run_object.first_hostrun()

    import_report(report_path, hostrun.id)
    rel_url = reverse('result-detail', args=(hostrun.result.id, ))
    return {'status': 'OK', 'url': request.build_absolute_uri(rel_url)}



from preupg.ui.config.models import AppSettings
from preupg.ui.report.forms import StateFilterForm

from django import template
from preupg.ui.utils.views import get_states_to_filter

register = template.Library()


@register.filter(name='state_filter_form')
def state_filter_form(result, get=None):
    states = get_states_to_filter(get)
    if states:
        form = StateFilterForm(get, result=result)
    else:
        init_states = AppSettings.get_initial_state_filter()
        inital_conf = {}
        for i in init_states:
            key = i + str(result.id)
            inital_conf[key] = True
        form = StateFilterForm(result=result, initial=inital_conf)
    return form
state_filter_form.is_safe = True



# -*- coding: utf-8 -*-

"""
This module serves for handling user interactions
"""

import os
import ConfigParser
import shutil
import sys

from distutils.util import strtobool
from preupg.utils import FileHelper
from preupg.creator import settings
from preupg import settings as preupgSettings

from preupg.settings import all_xccdf_xml_filename as ALL_XCCDF_XML


def get_user_input(message, default_yes=True, any_input=False):
    """
    Function for command line messages

    :param message: prompt string
    :param default_yes: If the default value is YES
    :param any_input: if True, return input without checking it first
    :return: True or False, based on user's input
    """
    choice = '[Y/n]'

    if any_input:
        msg = '{0} '.format(message)
    else:
        msg = '{0} {1}? '.format(message, choice)

    while True:
        if int(sys.version_info[0]) == 2:
            user_input = raw_input(msg)
        else:
            user_input = input(msg)

        if not user_input:
            if default_yes:
                return True
            else:
                return False
        if not any_input:
            try:
                user_input = strtobool(user_input)
            except ValueError:
                print('You have to type [y]es or [n]o.')
                continue

        return user_input


class UIHelper(object):

    def __init__(self, upgrade_path=None):
        self.content_dict = {}
        self.upgrade_path = upgrade_path
        self._group_name = ""
        self._content_name = ""
        self.content_path = None
        self.content_ini = ""
        self.check_script = True
        self.solution_file = True
        self.refresh_content = False
        self.script_type = None
        # properties.ini
        self.properties_ini_path = None
        self.src_version = None
        self.dst_version = None

    def _init_dict(self):
        self.content_dict['content_description'] = ''

    def get_group_name(self):
        return self._group_name

    def get_content_name(self):
        return self._content_name

    def _get_dict_value(self, name):
        try:
            return self.content_dict[name]
        except KeyError:
            return None

    def get_check_script(self):
        return preupgSettings.check_script

    def get_solution_file(self):
        return preupgSettings.solution_txt

    def get_content_ini_file(self):
        return preupgSettings.module_ini

    def get_content_path(self):
        if self.content_path is None:
            return None
        return os.path.join(self.get_upgrade_path(), self.content_path)

    def get_upgrade_path(self):
        if self.upgrade_path is None:
            return None
        if self.upgrade_path.startswith("/"):
            return self.upgrade_path
        return os.path.join(os.getcwd(), self.upgrade_path)

    def specify_upgrade_path(self):
        if self.upgrade_path is None:
            self.upgrade_path = UIHelper\
                .ask_mandatory_string(settings.upgrade_path,
                                      'The scenario is mandatory. '
                                      'You have to specify it.')

        message = 'The path %s already exists.\n'\
            'Do you want to create a module there?' % self.upgrade_path
        if UIHelper.check_path(self.get_upgrade_path(), message) is None:
            return None

        return True

    def prepare_content_env(self):
        self.content_path = os.path.join(self.get_group_name(),
                                         self.get_content_name())

    def get_properties_ini_info(self):
        """ If properties.ini file doesnt exist ask user for OS versions,
        otherwise don't ask anything
        """
        self.properties_ini_path = os.path.join(
            self.get_upgrade_path(),
            preupgSettings.properties_ini)
        if not self.properties_ini_exists():
            self.ask_for_properties_ini_versions()

    def properties_ini_exists(self):
        if self.properties_ini_path:
            return os.path.isfile(self.properties_ini_path)
        return False

    def ask_for_properties_ini_versions(self):
        """
        Asks user for src,dst OS versions, while options are mandatory user
        input is required
        """
        self.src_version = UIHelper.ask_mandatory_string(
            settings.prop_src_version,
            "The major source OS version is mandatory.")
        self.dst_version = UIHelper.ask_mandatory_string(
            settings.prop_dst_version,
            "The major destination OS version is mandatory.")

    def ask_for_script_type(self):
        while True:
            options = ['sh', 'py']
            self.script_type = get_user_input(settings.type_check_script, any_input=True)
            if self.script_type is True:
                self.script_type = "sh"
                break
            if self.script_type not in options:
                print("Select either 'sh or 'py'.")
                continue
            else:
                break

    def get_content_info(self):
        self._group_name = get_user_input(settings.group_name, any_input=True)
        if self._group_name is True:
            self._group_name = settings.default_group
        self._content_name = get_user_input(settings.content_name, any_input=True)
        if self._content_name is True:
            self._content_name = settings.default_module
        self.prepare_content_env()
        if os.path.exists(self.get_content_path()):
            message = "The module %s already exists.\n" \
                      "Do you want to overwrite it?" \
                      % os.path.join(self.upgrade_path,
                                     self.content_path)
            if UIHelper.check_path(os.path.join(self.upgrade_path,
                                                self.content_path),
                                   message):
                # User would like to overwrite the module
                self.refresh_content = True
        else:
            os.makedirs(self.get_content_path())

        self.ask_for_script_type()
        checkscript = self.get_check_script()
        if UIHelper.check_path(
                os.path.join(self.get_content_path(), checkscript),
                settings.check_path % checkscript) is None:
            self.check_script = False

        solution = self.get_solution_file()
        if UIHelper.check_path(os.path.join(self.get_content_path(), solution),
                               settings.check_path % solution) is None:
            self.solution_file = False

        self.content_dict['content_title'] = UIHelper\
            .ask_mandatory_string(settings.content_title,
                                  "Module title is mandatory!")

        self.content_dict['content_description'] = UIHelper\
            .ask_mandatory_string(settings.content_desc_text,
                                  "Module description is mandatory!")

    def _create_ini_file(self):
        """
        INI file should look like
        [preupgrade]
        content_title: Bacula Backup Software
        author: Petr Hracek <phracek@redhat.com>
        content_description: This module verifies the directory permissions for the Bacula service.
        applies_to: bacula-common
        :return:
        """
        config = ConfigParser.RawConfigParser()
        config.add_section(preupgSettings.prefix)
        for key, val in iter(self.content_dict.items()):
            if val is not None:
                config.set(preupgSettings.prefix, key, val)

        ini_path = os.path.join(self.get_content_path(),
                                self.get_content_ini_file())
        UIHelper.write_config_to_file(ini_path, config)

    def _create_check_script(self):
        if self.check_script:
            if self.script_type == "sh":
                content = settings.temp_bash_script
            else:
                content = settings.temp_python_script
            FileHelper.write_to_file(os.path.join(self.get_content_path(),
                                                  self.get_check_script()),
                                     'wb',
                                     content)
            os.chmod(os.path.join(self.get_content_path(),
                                  self.get_check_script()),
                     0755)

    def _create_solution_file(self):
        if self.solution_file:
            FileHelper.write_to_file(os.path.join(self.get_content_path(), self.get_solution_file()), 'wb', '')

    def _create_group_ini(self):
        """
        INI file should look like
        [preupgrade]
        group_title = <some title>
        :return:
        """
        config = ConfigParser.RawConfigParser()
        config.add_section(preupgSettings.prefix)
        config.set(preupgSettings.prefix, 'group_title', 'Title for %s ' % self.get_group_name())

        file_name = 'group.ini'
        group_ini = os.path.join(self.get_upgrade_path(),
                                 self.get_group_name(),
                                 file_name)
        if os.path.exists(group_ini):
            return
        UIHelper.write_config_to_file(group_ini, config)

    def create_properties_ini(self):
        """
        Create properties.ini inside module set directory in format:

        [preupgrade-assistant-modules]
        src_major_version = <user_input>
        dst_major_version = <user_input>
        """
        if not self.properties_ini_exists():
            section = 'preupgrade-assistant-modules'

            config = ConfigParser.RawConfigParser()
            config.add_section(section)
            config.set(section, 'src_major_version', self.src_version)
            config.set(section, 'dst_major_version', self.dst_version)

            UIHelper.write_config_to_file(self.properties_ini_path, config)

    def create_final_content(self):
        try:
            self.create_properties_ini()
            self._create_group_ini()
            self._create_ini_file()
        except IOError:
            return None

        self._create_check_script()
        self._create_solution_file()
        return True

    def _brief_summary(self):
        content_path = os.path.join(self.upgrade_path,
                                    self.get_group_name(),
                                    self.get_content_name())
        generated_all_xccdf_path = os.path.join(self.upgrade_path + '-results',
                                                ALL_XCCDF_XML)
        print(settings.summary_title)
        print(settings.summary_directory % self.get_content_path())
        print(settings.summary_ini
              % os.path.join(content_path, self.get_content_ini_file()))
        print(settings.summary_check
              % os.path.join(content_path, self.get_check_script()))
        print(settings.summary_solution
              % os.path.join(content_path, self.get_solution_file()))
        print(settings.commands_to_use_new_module
              % (self.upgrade_path, generated_all_xccdf_path))

    def take_manadatory_info(self):
        try:
            if self.specify_upgrade_path() is None:
                return 1

            self.get_properties_ini_info()

            self.get_content_info()
            if self.refresh_content:
                shutil.rmtree(self.get_content_path())
                os.makedirs(self.get_content_path())
            if self.create_final_content() is None:
                return 1
            self._brief_summary()
        except KeyboardInterrupt:
            if self.get_content_path() is not None:
                shutil.rmtree(self.get_content_path())
            print('\n Content creation was interrupted by user.\n')

    @staticmethod
    def check_path(path, msg):
        """Return None when the path exists and user answers No to the
        msg question.
        """
        if os.path.exists(path):
            choice = get_user_input(msg)
            if not choice:
                return None
        return True

    @staticmethod
    def is_valid_string(string):
        if isinstance(string, basestring) and bool(string.strip()):
            return True
        return False

    @staticmethod
    def ask_for_string(msg, err_msg):
        string = get_user_input(msg, any_input=True)
        if not UIHelper.is_valid_string(string):
            print(err_msg)
            return False
        return string

    @staticmethod
    def ask_mandatory_string(msg, err_msg):
        string = False
        while not string:
            string = UIHelper.ask_for_string(msg, err_msg)
        return string

    @staticmethod
    def write_config_to_file(file_path, config):
        """
        Create config file

        @param {str} file_path - path of new config file
        @param {RawConfigParser} config - configuration object with config data
        @throws {IOError}
        """
        try:
            with open(file_path, 'wb') as f:
                config.write(f)
        except IOError:
            print('An error occured while writing to the {0} file!'.format(
                file_path))
            raise




from __future__ import unicode_literals, print_function
import logging
import sys
from preupg import settings


class LoggerHelper(object):
    """
    Helper class for setting up a logger
    """

    @staticmethod
    def get_basic_logger(logger_name, level=logging.DEBUG):
        """
        Sets-up a basic logger without any handler
        :param logger_name: Logger name
        :param level: severity level
        :return: created logger
        """
        logger_name = logging.getLogger(logger_name)
        logger_name.setLevel(level)

        return logger_name

    @staticmethod
    def add_stream_handler(logger_name, level=None):
        """
        Adds console handler with given severity.
        :param logger_name: logger object to add the handler to
        :param level: severity level
        :return: None
        """
        console_handler = logging.StreamHandler(sys.stdout)
        if level:
            console_handler.setLevel(level)
        logger_name.addHandler(console_handler)

    @staticmethod
    def add_file_handler(logger_name, path, formatter=None, level=None):
        """
        Adds FileHandler to a given logger
        :param logger_name: Logger object to which the file handler will be
                            added
        :param path: Path to file where the debug log will be written
        :param formatter: logging format
        :param level: severity level
        :return: None
        """
        file_handler = logging.FileHandler(path, 'w')
        if level:
            file_handler.setLevel(level)
        if formatter:
            file_handler.setFormatter(formatter)
        logger_name.addHandler(file_handler)


logger = LoggerHelper.get_basic_logger('preupgrade-assistant')
logger_debug = LoggerHelper.get_basic_logger('preupgrade-assistant-debug')
logger_report = LoggerHelper.get_basic_logger('preupgrade-assistant-report',
                                              logging.DEBUG)


def log_message(message, new_line=True, level=logging.INFO):
    """ if verbose, log `msg % args` to stdout """
    if int(sys.version_info[0]) == 2:
        sys.stdout.write(message.encode(settings.defenc))
        sys.stdout.flush()
        # This is used in case that we do not want to print the new line
        if new_line:
            sys.stdout.write("\n")
            sys.stdout.flush()
    else:
        endline = "\n" if new_line else ""
        print(message, end=endline, file=sys.stdout, flush=True)

    logger_debug.log(level, message)




from __future__ import unicode_literals, print_function
import unittest
import os
import tempfile
import shutil

from preupg.kickstart.application import KickstartGenerator
from preupg.kickstart.cli import CLIKickstart
from preupg.kickstart.conf import ConfKickstart, DummyConfKickstart
from preupg import settings

try:
    import base
except ImportError:
    import tests.base as base


def get_full_path(file_name):
    return os.path.join(os.getcwd(), 'tests', 'kickstart_data', file_name)


class TestKickstartPartitioning(base.TestCase):

    kickstart = None
    dir_name = None
    WORKING_DIR = ''
    lsblk_list = 'lsblk_list'
    vgs_list = 'vgs_list'
    lvdisplay = 'lvdisplay'
    firewall_cmd = 'firewall-cmd'
    users = 'Users'
    groups = 'Groups'
    kg = None

    def setUp(self):
        ks_template = settings.KS_TEMPLATE
        self.WORKING_DIR = tempfile.mkdtemp(prefix='preupg')
        if os.path.isdir(self.WORKING_DIR):
            shutil.rmtree(self.WORKING_DIR)
        os.makedirs(self.WORKING_DIR)
        settings.KS_DIR = self.WORKING_DIR
        shutil.copyfile(os.path.join(os.getcwd(), 'data', 'templates',
                                     ks_template),
                        os.path.join(self.WORKING_DIR, ks_template))
        conf = {"y": True}
        dc = DummyConfKickstart(**conf)
        cli_kickstart = CLIKickstart(["--assumeyes"])
        conf = ConfKickstart(cli_kickstart.opts, dc, cli_kickstart)
        self.kg = KickstartGenerator(conf, self.WORKING_DIR,
                                     settings.KS_FILENAME)
        self.kg.collect_data()

    def test_lvm_partitions(self):
        lvm_lsblk = get_full_path('lvm_lsblk_list')
        vgs_list = get_full_path('vgs_list')
        lvdisplay = get_full_path('lvdisplay')
        shutil.copyfile(lvm_lsblk, os.path.join(self.WORKING_DIR, self.lsblk_list))
        shutil.copyfile(vgs_list, os.path.join(self.WORKING_DIR, self.vgs_list))
        shutil.copyfile(lvdisplay, os.path.join(self.WORKING_DIR, self.lvdisplay))
        self.kg.generate()
        expected_layout = ['clearpart --all',
                           'part /boot --ondisk=vda --size=500',
                           'part pv.01 --size=9000',
                           'volgroup vg_rhel67 --pesize=4096 pv.01',
                           'logvol /  --size=8000 --name=lv_root --vgname=vg_rhel67',
                           'logvol swap  --size=1000 --name=lv_swap --vgname=vg_rhel67']
        for layout in expected_layout:
            self.assertTrue(layout.strip(), self.kg.ks.handler.__str__())

    def test_lvm_crypt_partitions(self):
        lvm_lsblk = get_full_path('lvm_crypt_lsblk_list')
        vgs_list = get_full_path('vgs_list')
        lvdisplay = get_full_path('lvdisplay')
        shutil.copyfile(lvm_lsblk, os.path.join(self.WORKING_DIR, self.lsblk_list))
        shutil.copyfile(vgs_list, os.path.join(self.WORKING_DIR, self.vgs_list))
        shutil.copyfile(lvdisplay, os.path.join(self.WORKING_DIR, self.lvdisplay))
        self.kg.generate()
        expected_layout = ['clearpart --all',
                           'part /boot --ondisk=vda --size=500',
                           'part pv.01 --size=9000',
                           'volgroup vg_rhel67 --pesize=4096 pv.01',
                           'logvol /  --size=8000 --name=lv_root --vgname=vg_rhel67',
                           'logvol swap  --size=1000 --name=lv_swap --vgname=vg_rhel67']
        for layout in expected_layout:
            self.assertTrue(layout.strip(), self.kg.ks.handler.__str__())

    def test_crypt_partitions(self):
        lvm_lsblk = get_full_path('crypt_lsblk_list')
        shutil.copyfile(lvm_lsblk, os.path.join(self.WORKING_DIR, self.lsblk_list))
        self.kg.generate()
        expected_layout = ['clearpart --all',
                           'part / --size=3000 --encrypted',
                           'part /boot --ondisk=vda --size=200',
                           'part swap --ondisk=vda --size=2000']
        for layout in expected_layout:
            self.assertTrue(layout.strip(), self.kg.ks.handler.__str__())

    def test_raid_crypt_partitions(self):
        raid_lsblk = get_full_path('raid_lsblk_list')
        shutil.copyfile(raid_lsblk, os.path.join(self.WORKING_DIR, self.lsblk_list))
        self.kg.generate()
        expected_layout = ['clearpart --all',
                           'part /boot --ondisk=sda --size=200',
                           'part swap --ondisk=sda --size=1000',
                           'part raid.00001 --grow --size=2048',
                           'part raid.00002 --grow --size=2048',
                           'part raid.00003 --grow --size=2048',
                           'part raid.00004 --grow --size=2048',
                           'raid / --device=md1 --level=1 raid.00001 raid.00002',
                           'raid /home --device=md0 --level=0 --encrypted raid.00003 raid.00004'
                           ]
        for layout in expected_layout:
            self.assertTrue(layout.strip(), self.kg.ks.handler.__str__())

    def test_raid_second_partitions(self):
        raid_lsblk = get_full_path('raid_lsblk_second_list')
        shutil.copyfile(raid_lsblk, os.path.join(self.WORKING_DIR, self.lsblk_list))
        self.kg.generate()
        expected_layout = ['clearpart --all',
                           'part /boot --ondisk=vda --size=1000 ',
                           'part swap --ondisk=vdb --size=1000',
                           'part raid.00001 --grow --size=2048',
                           'part raid.00002 --grow --size=2048',
                           'raid / --device=md0 --level=0 raid.00001 raid.00002',
                           ]
        for layout in expected_layout:
            self.assertTrue(layout.strip(), self.kg.ks.handler.__str__())

    def test_native_partitioning(self):
        lsblk_native_lsblk = get_full_path('lsblk_native_list')
        shutil.copyfile(lsblk_native_lsblk, os.path.join(self.WORKING_DIR, self.lsblk_list))
        self.kg.generate()
        expected_layout = ['clearpart --all',
                           'part / --ondisk=vda --size=5000',
                           'part /boot --ondisk=vda --size=200',
                           'part /home --ondisk=vda --size=2000',
                           'part swap --ondisk=vda --size=1000',
                           ]
        for layout in expected_layout:
            self.assertTrue(layout.strip() in self.kg.ks.handler.__str__())

    def test_firewall_rules(self):
        firewall_cmd = get_full_path(self.firewall_cmd)
        shutil.copyfile(firewall_cmd, os.path.join(self.WORKING_DIR, self.firewall_cmd))
        self.kg.generate()
        expected_layout = ['firewall --enabled --service=foo,bar,test']
        for layout in expected_layout:
            self.assertTrue(layout.strip(), self.kg.ks.handler.__str__())

    def test_user_and_groups(self):
        files = ['Users', 'Groups', 'setup_passwd', 'uidgid']
        for f in files:
            shutil.copyfile(get_full_path(f), os.path.join(self.WORKING_DIR, f))
        self.kg.generate()
        expected_layout = ['group --name=foobar --gid=500',
                           'group --name=testfoo --gid=506',
                           'group --name=preupg --gid=501',
                           'user --homedir=/home/foobar --name=foobar --shell=/bin/bash --uid=500 --gid=500',
                           'user --homedir=/ --name=testfoo --shell=/sbin/nologin --uid=506 --gid=506',
                           'user --homedir=/home/preupg --name=preupg --shell=/sbin/nologin --uid=501 --gid=501']

        for layout in expected_layout:
            self.assertTrue(layout.strip(), self.kg.ks.handler.__str__())

    def tearDown(self):
        pass


class TestKickstartFirewall(object):

    def setUp(self):
        pass

    def tearDown(self):
        pass


def suite():
    loader = unittest.TestLoader()
    suite = unittest.TestSuite()
    suite.addTest(loader.loadTestsFromTestCase(TestKickstartPartitioning))
    return suite

if __name__ == '__main__':
    unittest.TextTestRunner(verbosity=3).run(suite())
